package did.gui;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.text.*;
import java.awt.event.*;
import javax.swing.tree.TreePath;

import org.w3c.dom.Node;

import did.gui.DIDMultTextArea;
import did.xml.DIDNode;
import did.xml.DIDDefaultTreeModel;

/**************************************************************************
*<pre>
*File:         DIDFindMsgBox.java
*Version:      1.0
*Description:  This class provides an ability to search the ground truth text
* to find entities that contains a keyword string. Because the ground truth text
* may have errors, the approximate string matching with an edit distance is 
* provided. All entities having a keyword string are highlighted after the 
* search is finished.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX         xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
***************************************************************************/

public class DIDFindMsgBox extends JFrame implements ActionListener {
	/** Reference to the TreeModel for which this class is acting as a viewer. */
	protected DIDDefaultTreeModel	myModel		= null;
	protected DIDImageCanvas	myCanvas	= null;

	private JPanel panel;
	private JPanel textPanel;
	private JPanel thresoldPanel;
	private JPanel buttonPanel;
	private DIDMultTextArea textSearchStr;
	private JTextField textThresold;
	private JLabel labSearch;
	private JLabel labThresold;
	private JButton btnSearch;
	private JButton btnClear;
	private JButton btnReset;
	private JButton btnClose;

	private int D[][];
	private int threshold;
	private String text;
	private String pattern;

	private DIDUnicodeTable myUnicodeTable = null;
	private DIDInputMethod myInputMethod = null;

	// Menu
	private JMenuBar myMenuBar = new JMenuBar();
	private static final String KEYBOARD = "Keyboard",
			KEYBOARDTAB = "Unicode Table", KEYBOARDENG = "English",
			KEYBOARDJAP = "Japanese", KEYBOARDRUS = "Russian",
			KEYBOARDKOR = "Korean";
	private static final String VIEW = "View", VIEWKEYB = "View Keyboard";
	private JMenu mnuKeyboard, mnuView;
	private JMenuItem mniKeyboardTab;
	private JRadioButtonMenuItem mniKeyboardEng, mniKeyboardJap,
			mniKeyboardKor, mniKeyboardRus;
	private JMenuItem mniViewKeyB;


	public DIDFindMsgBox(DIDImageCanvas canvas, DIDDefaultTreeModel model) {
		super("Search");
		myCanvas = canvas;
		myModel = model;
		//setSize(400, 200);
		setLocation(100,100);
		setJMenuBar(buildMenuBar());

		panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

		textSearchStr = new DIDMultTextArea();
		textSearchStr.setBorder(BorderFactory.createLineBorder(Color.darkGray, 1));
		int fontsize = (int)(textSearchStr.getFont().getSize()*1.5);
		textSearchStr.setFont(new Font(textSearchStr.getFont().getName(), textSearchStr.getFont().getStyle(), fontsize));
		textSearchStr.setColumns(20);

		labSearch = new JLabel("Search String:");

		textThresold = new JTextField("2");
		textThresold.setColumns(5);

		labThresold = new JLabel("Edit Distance:");
		btnSearch = new JButton("Search");
		btnSearch.addActionListener(new ButtonListener());
		btnClear = new JButton("Clear");
		btnClear.addActionListener(new ButtonListener());
		btnReset = new JButton("Reset");
		btnReset.addActionListener(new ButtonListener());
		btnClose = new JButton("Close");
		btnClose.addActionListener(new ButtonListener());

		textPanel = new JPanel();
		textPanel.setLayout(new BoxLayout(textPanel, BoxLayout.X_AXIS));
		textPanel.add(Box.createRigidArea(new Dimension(8, 1)));
		textPanel.add(labSearch);
		textPanel.add(Box.createRigidArea(new Dimension(30, 1)));
		textPanel.add(textSearchStr);

		thresoldPanel = new JPanel();
		thresoldPanel.setLayout(new BoxLayout(thresoldPanel, BoxLayout.X_AXIS));
		thresoldPanel.add(Box.createRigidArea(new Dimension(8, 1)));
		thresoldPanel.add(labThresold);
		thresoldPanel.add(Box.createRigidArea(new Dimension(30, 1)));
		thresoldPanel.add(textThresold);

		buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(btnSearch);
		buttonPanel.add(btnClear);
		buttonPanel.add(btnReset);
		buttonPanel.add(btnClose);

		panel.add(textPanel);
		panel.add(thresoldPanel);
		panel.add(buttonPanel);

		getContentPane().add(panel); 
		pack();
		hide();
	}

	private JMenuBar buildMenuBar() {
		mnuKeyboard = new JMenu(KEYBOARD);
		mniKeyboardTab = new JMenuItem(KEYBOARDTAB);
		mniKeyboardTab.addActionListener(this);
		mnuKeyboard.add(mniKeyboardTab);
		mnuKeyboard.addSeparator();

		mniKeyboardEng = new JRadioButtonMenuItem(KEYBOARDENG);
		mniKeyboardEng.addActionListener(this);
		mnuKeyboard.add(mniKeyboardEng);
		mniKeyboardJap = new JRadioButtonMenuItem(KEYBOARDJAP);
		mniKeyboardJap.addActionListener(this);
		mnuKeyboard.add(mniKeyboardJap);
		mniKeyboardKor = new JRadioButtonMenuItem(KEYBOARDKOR);
		mniKeyboardKor.addActionListener(this);
		mnuKeyboard.add(mniKeyboardKor);
		mniKeyboardRus = new JRadioButtonMenuItem(KEYBOARDRUS);
		mniKeyboardRus.addActionListener(this);
		mnuKeyboard.add(mniKeyboardRus);

		ButtonGroup keybGroup = new ButtonGroup();
		keybGroup.add(mniKeyboardEng);
		keybGroup.add(mniKeyboardJap);
		keybGroup.add(mniKeyboardKor);
		keybGroup.add(mniKeyboardRus);
		mniKeyboardEng.setSelected(true);
		/*DEL*/mniKeyboardJap.setEnabled(false);
		/*DEL*/mniKeyboardKor.setEnabled(false);
		myMenuBar.add(mnuKeyboard);

		mnuView = new JMenu(VIEW);
		mniViewKeyB = new JMenuItem(VIEWKEYB);
		mniViewKeyB.addActionListener(this);
		mnuView.add(mniViewKeyB);
		myMenuBar.add(mnuView);

		return myMenuBar;
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == KEYBOARDTAB) {
			myUnicodeTable = new DIDUnicodeTable(textSearchStr);
			myUnicodeTable.show();
		} else if (e.getActionCommand() == KEYBOARDENG) {
			myInputMethod = null;
			textSearchStr.setInputMethod(myInputMethod);
		} else if (e.getActionCommand() == KEYBOARDJAP) {
		} else if (e.getActionCommand() == KEYBOARDKOR) {
		} else if (e.getActionCommand() == KEYBOARDRUS) {
			myInputMethod = new DIDRussianInputMethod(false);
			textSearchStr.setInputMethod(myInputMethod);
		} else if (e.getActionCommand() == VIEWKEYB) {
			if (myInputMethod != null) {
				myInputMethod.showKeyboard();
			}
		}
	}



	public boolean levelSearch(String level)  {
		int matchNum = 0;
		DIDNode entity = null;
		int k = Integer.parseInt(textThresold.getText());

		try {
			for (Enumeration enu = ((DIDNode)myModel.getRoot()).preorderEnumeration() ; enu.hasMoreElements() ;) {
			DIDNode tmpNode = (DIDNode)enu.nextElement();
				if (tmpNode.getDataType().equals(tmpNode.getNodeName())) {
					entity = tmpNode;
				}
				if(tmpNode.getNodeName().equals("GT_Text")) {
					tmpNode = (DIDNode)enu.nextElement();
					while(!(tmpNode.getNodeName().equals("Value"))) {
						tmpNode = (DIDNode)enu.nextElement();
					}
					if(tmpNode.getNodeName().equals("Value")) {
						int num = approxMatch(textSearchStr.getText(), tmpNode.getNodeValue(), k);
						if(num > 0 && entity.getDataType().equals(level))  {
							matchNum++;
							entity.getRoi().setSelected(true);
							myCanvas.myTree.setSelectionPath(new TreePath(entity.getPath()));
							/*DEL*/ if(entity != null) System.out.print("Found "+textSearchStr.getText()+" in Entity: "+entity.getNodeName());
							/*DEL*/ System.out.println(" Text: "+tmpNode.getNodeValue());
						}
					}
				}
			}
			if(matchNum > 0)  return true;
		/*DEL*/  System.out.println("found "+matchNum);
		} catch (Exception enu) { }

		return false;
	}

	class ButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JComponent c = (JComponent) e.getSource();

				if(c == btnSearch) {
					myCanvas.resetFocus();
					myCanvas.setMode(DIDImageCanvas.EDIT);
					myCanvas.setView(DIDRoiIFC.PAGE);
					myCanvas.setFillColors(false);
					myCanvas.repaint();
					for (int i=myModel.EntityType.length()-1; i >= 0; i--) {
						if (levelSearch(myModel.EntityType.getName(i))) {
							break;
						}
					}

				} else if(c == btnClear) {
					textSearchStr.setText("");
				} else if(c == btnReset) {
					myCanvas.resetFocus();
					myCanvas.repaint();
				} else if(c == btnClose) {
					if (myInputMethod != null && myInputMethod.getKeyBDisplay() != null) {
						myInputMethod.getKeyBDisplay().dispose()
;
					}
					if (myUnicodeTable != null) {
						myUnicodeTable.dispose();
					}

					myCanvas.resetFocus();
					myCanvas.repaint();
					hide();
				}
		}
	}

/*************************************************************
	*  Approximate String Matching
	************************************************************/
	public void initApproxMatch(String p, String t, int k) {
		pattern = new String(p);
		text = new String(t);
		D = new int[pattern.length()+1][pattern.length()+k+1];
		threshold = k;
	}

	public int approxMatch(String p, String t, int k)  {
		int matchNum = 0;

		initApproxMatch(p, t, k);
		for(int i=0; i<=(text.length()-pattern.length()-threshold); i++)  {
			if(subApproxMatch(pattern, text.substring(i, i+pattern.length()+threshold)))
				matchNum++;
		}
		int startPos = text.length()-pattern.length()-threshold+1;
		startPos = (startPos > 0)? startPos : 0;
		for(int i=startPos; i<=(text.length()-pattern.length()+threshold); i++) {
			if(subApproxMatch(pattern, text.substring(i, text.length())))
				matchNum++;
		}
		return matchNum;
	}

	public boolean subApproxMatch(String P, String T)  {
		int i, j, p;
		int ADD, SUB, SWAP;

		// Initialize D
		for(i = 0; i <= P.length(); i++)
			D[i][0] = i;
		for(j = 0; j <= T.length(); j++)
			D[0][j] = j;

		// Calculate error of matched string ending
		for (j = 1; j <= T.length(); j++) {
			for(i = 1; i <= P.length(); i++) {
				if(P.charAt(i-1) == T.charAt(j-1))
					p = 0;
				else
					p = 1;

				// Assign a value to D[i][j] sugh that
				// D[i][j] = min{D[i-1][j] + 1,
				// D[i][j-1] + 1,
				// D[i-1][j-1] + p}
				SUB = D[i][j-1] + 1;
				ADD = D[i-1][j] + 1;
				SWAP = D[i-1][j-1] + p;
				if((SUB < SWAP) && (SUB < ADD)) {
					D[i][j] = SUB;
				}
				else if((ADD < SUB) && (ADD < SWAP)) {
					D[i][j] = ADD;
				}
				else {
					D[i][j] = SWAP;
				}
			}

			// if(D[P.length()][j] <= k)
			// An acceptable string ends at j
		}
		int minidx = 0;
		minidx = T.length()-2*threshold;
		minidx = (minidx > 0)? minidx: 0;
		for(i=minidx; i<= T.length(); i++) {
			if(D[P.length()][i] < D[P.length()][minidx]) {
				minidx = i;
			}
		}
		if(D[P.length()][minidx] <= threshold) {
			return true;
		} else {
			return false;
		}
	}


}
