package did.gui;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.swing.JTextArea;

import did.gui.DIDInputMethod;

public class DIDMultTextArea extends JTextArea implements KeyListener {

	DIDInputMethod myInputMethod = null;

	/**
	* Constructs a DIDMultTextArea.
	* @param enableInputMethods whether to enable input methods for this component
	*/
	public DIDMultTextArea() {
		super();
		setVisible(true);
		setEnabled(true);
		addKeyListener(this);
		enableInputMethods(true);
	}

	public DIDMultTextArea(String str) {
		super(str);
		setVisible(true);
		setEnabled(true);
		addKeyListener(this);
		enableInputMethods(true);
	}

	public void setInputMethod(DIDInputMethod im) {
		myInputMethod = im;
	}
    
	public void keyTyped(KeyEvent event) {
		char keyChar = event.getKeyChar();
		if (myInputMethod == null && keyChar != '\b'
			&& keyChar != '\n' && keyChar != (char)0x7f) {
			replaceRange("", getSelectionStart(), getSelectionEnd());
			insert((new Character(keyChar)).toString(), getCaretPosition());
		} else if (myInputMethod != null && keyChar != '\n'
				&& keyChar != (char)0x7f) {
			replaceRange("", getSelectionStart(), getSelectionEnd());
			myInputMethod.keyTyped(keyChar);
			String composingText = myInputMethod.getComposingText();
			if (composingText != null && composingText.length() > 0) {
				int start = getCaretPosition();
				insert(composingText, getCaretPosition());
				int end = getCaretPosition();
				setSelectionStart(start);
				setSelectionEnd(end);
			} else {
				insert(myInputMethod.getCommittedText(),
					getCaretPosition());
			}
		}
		event.consume();
		repaint();
	}

	/** Ignores key pressed events. */
	public void keyPressed(KeyEvent event) {}

	/** Ignores key released events. */
	public void keyReleased(KeyEvent event) {}
    
}
