package did.gui;

import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Rectangle;

import javax.swing.JPanel;
import javax.swing.JComponent;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;
import javax.swing.JTextArea;

public class DIDOcrCanvas extends JComponent implements Scrollable {

	/** An integer representing the maximum unit the component should be scrolled (when necessary). */
	protected int maxUnitIncrement = 1;

	public DIDOcrCanvas() {
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		g.drawString("OCR Image Canvas", 200, 200);
	}

	/*****************************************************************************************
	* Part of Scrollable Interface Implementation used to return the preferred display size 
	* of the ImageCanvas.
	* @return Dimension This method returns a Dimension object represention the perferred size
	* of the this object.
	*****************************************************************************************/
	public Dimension getPreferredScrollableViewportSize() { 
		return getPreferredSize();
	}

	/*****************************************************************************************
	* Part of Scrollable Interface Implementation necessary to determine the amount by which 
	* to scroll the object in the viewport when necessary.
	* @return This method returns an integer represeting the number of pixels to scroll.
	* @param visibleRect A Rectangle object representing the visible portion of the component.
	* @param orientation Either SwingConstants.VERTICAL or SwingConstants.HORIZONTAL
	* @param direction Less than zero to scroll up/left, greater than zero for down/right.
	*****************************************************************************************/
	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
		int currentPosition = 0;
		if (orientation == SwingConstants.HORIZONTAL) { 
			currentPosition = visibleRect.x;
		} else {
			currentPosition = visibleRect.y;
		}
		if (direction < 0) {
			int newPosition = currentPosition - 
				(currentPosition / maxUnitIncrement) * maxUnitIncrement;
			return (newPosition == 0) ? maxUnitIncrement : newPosition;
		} else { 
			return ((currentPosition / maxUnitIncrement) + 1) *
				maxUnitIncrement - currentPosition;
		}
	}
	
	/*****************************************************************************************
	* Method used to determine the amount for a block inrecment, which is the height or width 
	* of visibleRect, based on orientation.
	* @param visibleRect The view area visible within the viewport 
	* @param orientation Either SwingConstants.VERTICAL or SwingConstants.HORIZONTAL
	* @param direction Less than zero to scroll up/left, greater than zero for down/right.
	* @return Returns the amount for a block inrecment, which is the height or width of 
	* visibleRect, based on orientation
	*****************************************************************************************/   
	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) { 
		if (orientation == SwingConstants.HORIZONTAL) { 
			return visibleRect.width - maxUnitIncrement;
		} else {
			return visibleRect.height - maxUnitIncrement;
		}
	}

	/*****************************************************************************************
	* Returns false to indicate that the height of the viewport does not determine the height 
	* of the table, unless the preferred height of the tree is smaller than the viewports 
	* height. In other words: ensure that the tree is never smaller than its viewport.
	* @return false
	*****************************************************************************************/   
	public boolean getScrollableTracksViewportHeight() {
		return false;
	}
	
	/*****************************************************************************************
	* Returns false to indicate that the width of the viewport does not determine the width of
	* the table, unless the preferred width of the tree is smaller than the viewports width. 
	* In other words: ensure that the tree is never smaller than its viewport.
	* @return false
	*****************************************************************************************/   
	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	/*****************************************************************************************
	* Method used to set the maximum unit (in pixels) by which this object should scroll.  
	* @param pixels An integer representing the number of pixels to scroll by when scrolling
	* is called necessary.
	*****************************************************************************************/   
	public void setMaxUnitIncrement(int pixels) {
		maxUnitIncrement = pixels;
	}
}
