package did.gui;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;

/*****************************************************************************************
*<pre>
*File:         DIDProperties.java
*Version:      1.1
*Description:  An extension of the Java Properties class implmented as a singleton.  This
*class handles all the user configurable settings for this application.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
******************************************************************************************/

public class DIDProperties extends Properties {

	/** Attribute referencing the sole instantiation of this class. */
	private static DIDProperties instance = null;

	/******************************************************************************************
	* This method is used to instantiate the class.  Since this class is a singleton the only 
	* way this method can be invoked is through the getInstance() method.  This constructor
	* is responsible for loading this class from a properties file.
	* @see #getInstance()
	******************************************************************************************/
	private DIDProperties() {
		try	{
			FileInputStream	myInput = new FileInputStream(new String("trueviz.properties"));
			load(myInput);
			myInput.close();
		} catch (Exception e) {
			System.out.println("Could not load properties file.  Error: " + e);
		}
	}

	/******************************************************************************************
	* This method is used to retrieve the only instance of this class stored locally as an 
	* attribute.  If the attribute is null the constructor is called and a new instance is 
	* created.
	* @return An instance of this class.
	* @see #instance
	******************************************************************************************/
	public static DIDProperties getInstance() {
		if (instance == null) {
			instance = new DIDProperties();
		}
		return instance;
	}
}
