package did.gui;

import did.gui.DIDInputMethod;

public class DIDRussianInputMethod implements DIDInputMethod {
	protected StringBuffer composingText = new StringBuffer();
	protected String committedText = new String();
	protected boolean committed = false;
	public DIDKeyBDisplay myKeyboard = null;

	private char UpperMap[] = {
		'\u0424', '\u0418', '\u0421', '\u0412', '\u0423',
		'\u0410', '\u041F', '\u0420', '\u0428', '\u041E',
		'\u041B', '\u0414', '\u042C', '\u0422', '\u0429',
		'\u0417', '\u0419', '\u041A', '\u042B', '\u0415',
		'\u0413', '\u041C', '\u0426', '\u0427', '\u041D',
		'\u042F'
		};

	private char SpecialKey[] = {
		'{', '}', ':', '"', '<', // with shift key
		'>', '?',
		'[', ']', ';', '\'', ',', // without shift key
		'.', '/',
		(char)0 };
	private char SpecialMap[] = {
		'\u0425', '\u042A', '\u0416', '\u042d', '\u0411',
		'\u042E', '\u0401',
		'\u0445', '\u044A', '\u0436', '\u044d', '\u0431',
		'\u044E', '\u0421'
		};

	public DIDRussianInputMethod(boolean showkbd) {
		if (showkbd) showKeyboard();
	}

	public String getComposingText() {
		return composingText.toString();
	}

	public String getCommittedText() {
		return committedText;
	}

	private int isSpecialChar(char ch) {
		for (int i=0; SpecialKey[i] != 0; i++) {
			if (SpecialKey[i] == ch) {
				return i;
			}
		}
		return -1;
	}

	public void keyTyped(char ch) {
		int index = -1;

		if (ch >= 'A' && ch <= 'Z') {
			composingText.append((char)(UpperMap[ch-'A']));
			committed = true;
		} else if (ch >= 'a' && ch <= 'z') {
			composingText.append((char)(UpperMap[ch-'a'] + 0x20));
			committed = true;
		} else if ((index = isSpecialChar(ch)) >= 0){
			composingText.append((char)(SpecialMap[index]));
			committed = true;
		} else if (ch == '\b'){
			int len = composingText.length();
			if (len > 0) {
				composingText.setLength(len - 1);
			} else {
				
			}
		} else {
			composingText.append(ch);
			committed = true;
		}

		if (committed) {
			committedText = composingText.toString();
			composingText.delete(0, composingText.length());
			committed = false;
		} else {
			committedText = new String();
		}
	}

	public DIDKeyBDisplay getKeyBDisplay() {
		return myKeyboard;
	}

	public void showKeyboard() {
		int index;
		myKeyboard = new DIDKeyBDisplay();

		char row1[] = myKeyboard.orgRow1;
		char row2[] = myKeyboard.orgRow2;
		char row3[] = myKeyboard.orgRow3;
		char row4[] = myKeyboard.orgRow4;

		myKeyboard.currRow1 = new char[14];
		myKeyboard.currRow2 = new char[12];
		myKeyboard.currRow3 = new char[11];
		myKeyboard.currRow4 = new char[10];

		myKeyboard.shiftRow1 = new char[14];
		myKeyboard.shiftRow2 = new char[12];
		myKeyboard.shiftRow3 = new char[11];
		myKeyboard.shiftRow4 = new char[10];
/*===>
		for (int i=0; i<14; i++) {
			if (ch >= 'A' && ch <= 'Z') {
			} else if ((index = isSpecialChar(ch)) >= 0){
			}
		}
<===*/
		for (int i=0; i<12; i++) {
			if (row2[i] >= 'A' && row2[i] <= 'Z') {
				myKeyboard.shiftRow2[i] = UpperMap[row2[i]-'A'];
				myKeyboard.currRow2[i] = (char)(UpperMap[row2[i]-'A']+0x20);
			} else if ((index = isSpecialChar(row2[i])) >= 0){
				myKeyboard.currRow2[i] = SpecialMap[index];
				myKeyboard.shiftRow2[i] = (char)(SpecialMap[index]-0x20);
			}
		}

		for (int i=0; i<11; i++) {
			if (row3[i] >= 'A' && row3[i] <= 'Z') {
				myKeyboard.shiftRow3[i] = UpperMap[row3[i]-'A'];
				myKeyboard.currRow3[i] = (char)(UpperMap[row3[i]-'A']+0x20);
			} else if ((index = isSpecialChar(row3[i])) >= 0){
				myKeyboard.currRow3[i] = SpecialMap[index];
				myKeyboard.shiftRow3[i] = (char)(SpecialMap[index]-0x20);
			}
		}

		for (int i=0; i<10; i++) {
			if (row4[i] >= 'A' && row4[i] <= 'Z') {
				myKeyboard.shiftRow4[i] = UpperMap[row4[i]-'A'];
				myKeyboard.currRow4[i] = (char)(UpperMap[row4[i]-'A']+0x20);
			} else if ((index = isSpecialChar(row4[i])) >= 0){
				myKeyboard.currRow4[i] = SpecialMap[index];
				myKeyboard.shiftRow4[i] = (char)(SpecialMap[index]-0x20);
			}
		}

		myKeyboard.setCurrs();
		myKeyboard.setShifts();
		myKeyboard.show();
	}
}
