package did.gui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

import did.gui.DIDUnicodeTable;
import did.gui.DIDInputMethod;
import did.gui.DIDRussianInputMethod;
import did.xml.DIDNode;

public class DIDUnicodeEditor extends JFrame implements ActionListener {
	private String originalText;
        private JScrollPane scrollPane;
	private JPanel buttonPanel;
	private DIDMultTextArea readText;
	private JButton btnUpdate, btnCancel, btnClose;
	private DIDNode myNode;
	private DIDUnicodeTable myUnicodeTable = null;
	private DIDInputMethod myInputMethod = null;

	// Menu
	private JMenuBar myMenuBar = new JMenuBar();
	private static final String KEYBOARD = "Keyboard",
		KEYBOARDTAB = "Unicode Table", KEYBOARDENG = "English",
		KEYBOARDJAP = "Japanese", KEYBOARDRUS = "Russian",
		KEYBOARDKOR = "Korean";
	private static final String VIEW = "View", VIEWKEYB = "View Keyboard";
	private JMenu mnuKeyboard, mnuView;
	private JMenuItem mniKeyboardTab;
	private JRadioButtonMenuItem mniKeyboardEng, mniKeyboardJap,
			mniKeyboardKor, mniKeyboardRus;
	private JMenuItem mniViewKeyB;

	public DIDUnicodeEditor(DIDNode node) {
		super("Ground Truth Text Editor");
		setSize(400, 400);

		setJMenuBar(buildMenuBar());

		myNode = node;
		originalText = node.getNodeValue();
		String str = node.getNodeValue();

		String displayString = new String();
		displayString += str;

		readText = new DIDMultTextArea(displayString);
		scrollPane = new JScrollPane(readText);

		btnUpdate = new JButton("Update");
		btnUpdate.addActionListener(new ButtonListener());
		btnCancel = new JButton("Cancel");
		btnCancel.addActionListener(new ButtonListener());
		btnClose = new JButton("Close");
		btnClose.addActionListener(new ButtonListener());

		buttonPanel = new JPanel();
		buttonPanel.add(btnUpdate);
		buttonPanel.add(btnCancel);
		buttonPanel.add(btnClose);

		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(scrollPane, BorderLayout.CENTER);
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);

		show();
	}


	private JMenuBar buildMenuBar() {
		mnuKeyboard = new JMenu(KEYBOARD);
		mniKeyboardTab = new JMenuItem(KEYBOARDTAB);
		mniKeyboardTab.addActionListener(this);
		mnuKeyboard.add(mniKeyboardTab);
		mnuKeyboard.addSeparator();

		mniKeyboardEng = new JRadioButtonMenuItem(KEYBOARDENG);
		mniKeyboardEng.addActionListener(this);
		mnuKeyboard.add(mniKeyboardEng);
		mniKeyboardJap = new JRadioButtonMenuItem(KEYBOARDJAP);
		mniKeyboardJap.addActionListener(this);
		mnuKeyboard.add(mniKeyboardJap);
		mniKeyboardKor = new JRadioButtonMenuItem(KEYBOARDKOR);
		mniKeyboardKor.addActionListener(this);
		mnuKeyboard.add(mniKeyboardKor);
		mniKeyboardRus = new JRadioButtonMenuItem(KEYBOARDRUS);
		mniKeyboardRus.addActionListener(this);
		mnuKeyboard.add(mniKeyboardRus);

		ButtonGroup keybGroup = new ButtonGroup();
		keybGroup.add(mniKeyboardEng);
		keybGroup.add(mniKeyboardJap);
		keybGroup.add(mniKeyboardKor);
		keybGroup.add(mniKeyboardRus);
		mniKeyboardEng.setSelected(true);
		/*DEL*/mniKeyboardJap.setEnabled(false);
		/*DEL*/mniKeyboardKor.setEnabled(false);
		myMenuBar.add(mnuKeyboard);

		mnuView = new JMenu(VIEW);
		mniViewKeyB = new JMenuItem(VIEWKEYB);
		mniViewKeyB.addActionListener(this);
		mnuView.add(mniViewKeyB);
		myMenuBar.add(mnuView);

		return myMenuBar;
	}

	static String readInput(String filename, String encoding) {
		StringBuffer buffer = new StringBuffer();
		try {
			FileInputStream hfis = new FileInputStream(filename);
			InputStreamReader hisr = new InputStreamReader(hfis, encoding);
			// or
			String defaultEncoding = hisr.getEncoding();
			Reader in = new BufferedReader(hisr);
			int ch;
			while ((ch = in.read()) > -1) {
				buffer.append((char)ch);
			}
			in.close();
			return buffer.toString();
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == KEYBOARDTAB) {
			myUnicodeTable = new DIDUnicodeTable(readText);
			myUnicodeTable.show();
		} else if (e.getActionCommand() == KEYBOARDENG) {
			myInputMethod = null;
			readText.setInputMethod(myInputMethod);
		} else if (e.getActionCommand() == KEYBOARDJAP) {
		} else if (e.getActionCommand() == KEYBOARDKOR) {
		} else if (e.getActionCommand() == KEYBOARDRUS) {
			myInputMethod = new DIDRussianInputMethod(false);
			readText.setInputMethod(myInputMethod);
		} else if (e.getActionCommand() == VIEWKEYB) {
			if (myInputMethod != null) {
				myInputMethod.showKeyboard();
			}
		}
	}


	/**  */
	class ButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JComponent c = (JComponent) e.getSource();

			if(c == btnUpdate) {
				myNode.setNodeValue(readText.getText());
			} else if(c == btnCancel) {
				readText.setText(originalText);
				myNode.setNodeValue(readText.getText());
			} else if(c == btnClose) {
				if (myInputMethod != null && myInputMethod.getKeyBDisplay() != null) {
					myInputMethod.getKeyBDisplay().dispose();
				}
				if (myUnicodeTable != null) {
					myUnicodeTable.dispose();
				}
				dispose();
			}
		}
	} // end of class ButtonListener
}
