package did.gui;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.event.*;
import javax.swing.tree.TreePath;

/**************************************************************************
*<pre>
*File:         DIDUnicodeTable.java
*Version:      1.0
*Description:  This class provides an ability to search and input the unicode
* character.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX         xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
***************************************************************************/

public class DIDUnicodeTable extends JFrame {
	public static final int RADIX = 16;
	protected int initVal = 0;
	protected int selectedRow, selectedCol;

	private JTextArea myArea = null;

	private JPanel tablePanel;
	private JPanel topPanel;
	private JPanel bottomPanel;

	private JPanel inputPanel;
	private JPanel codePanel;
	private JPanel buttonPanel;
	private JTextField textString;
	private JLabel labString;
	private JTextField textCode;
	private JLabel labCode;
	private JButton btnSelect;
	private JButton btnClear;
	private JButton btnInsert;
	private JButton btnClose;

	JComboBox cmbLang;
	JTextField textPage;
	private JButton btnUp;
	private JButton btnDown;
	private JButton btnGo;

	public DIDUnicodeTable(JTextArea textArea) {
		super("Unicode Table");
		myArea = textArea;

		setSize(400, 200);
		setLocation(100,100);
		tablePanel = new JPanel();
		topPanel = new JPanel();
		bottomPanel = new JPanel();
		bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.Y_AXIS));
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(topPanel, BorderLayout.NORTH);
		getContentPane().add(tablePanel, BorderLayout.CENTER);
		getContentPane().add(bottomPanel, BorderLayout.SOUTH);

		// build topPanel
		cmbLang = new JComboBox();
		cmbLang.addItem("English");
		cmbLang.addItem("Japanese");
		cmbLang.addItem("Korean");
		cmbLang.addItem("Russian");
		cmbLang.addItemListener(new ComboListener());
		btnDown = new JButton("Previous");
		btnDown.addActionListener(new ButtonListener());
		textPage = new JTextField();
		textPage.setText(getHexStr(initVal));
		textPage.setColumns(4);
		btnUp = new JButton("Next");
		btnUp.addActionListener(new ButtonListener());
		btnGo = new JButton("Go");
		btnGo.addActionListener(new ButtonListener());
		topPanel.add(cmbLang);
		topPanel.add(btnDown);
		topPanel.add(textPage);
		topPanel.add(btnUp);
		topPanel.add(btnGo);

		// build tablePanel
		tablePanel.add(createTable());

		// build bottomPanel
		textString = new JTextField();
		int fontsize = (int)(textString.getFont().getSize()*1.5);
		Font newFont = new Font(textString.getFont().getName(),
				textString.getFont().getStyle(), fontsize);
		textString.setFont(newFont);
		textString.setColumns(20);

		textCode = new JTextField();
		//int fontsize = (int)(textCode.getFont().getSize()*1.5);
		//textCode.setFont(newFont);
		textCode.setColumns(4);

		labString = new JLabel(" Text: ");
		labCode = new JLabel(" Unicode: ");

		btnSelect = new JButton("Select");
		btnSelect.addActionListener(new ButtonListener());
		btnInsert = new JButton("Insert");
		btnInsert.addActionListener(new ButtonListener());
		btnClear = new JButton("Clear");
		btnClear.addActionListener(new ButtonListener());
		btnClose = new JButton("Close");
		btnClose.addActionListener(new ButtonListener());

		JPanel lpanel = new JPanel();
		JPanel tpanel = new JPanel();
		lpanel.setLayout(new GridLayout(2,1));
		tpanel.setLayout(new GridLayout(2,1));

		lpanel.add(labString);
		lpanel.add(labCode);
		tpanel.add(textString);
		tpanel.add(textCode);

		inputPanel = new JPanel();
		inputPanel.setLayout(new BoxLayout(inputPanel, BoxLayout.X_AXIS));
		inputPanel.add(lpanel);
		inputPanel.add(tpanel);

		buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(btnSelect);
		buttonPanel.add(btnInsert);
		buttonPanel.add(btnClear);
		buttonPanel.add(btnClose);

		bottomPanel.add(inputPanel);
		bottomPanel.add(buttonPanel);

		pack();
	}

	public JTable createTable() {
		TableModel dataModel = new AbstractTableModel() {
			public int getColumnCount() { return RADIX; }
			public int getRowCount() { return RADIX;}
			public String getColumnName(int column) {
				return ((new Integer(column)).toString());
			}
			public Class getColumnClass(int c) {
				return getValueAt(0, c).getClass();
			}
			public Object getValueAt(int row, int col) {
				int codeVal = initVal + row*RADIX + col;
				return (new Character((char)codeVal));
			}
		};

		JTable table = new JTable(dataModel);
		table.setColumnSelectionAllowed(false);
		table.setRowSelectionAllowed(false);

		TableColumn column = null;
		for (int i = 0; i < RADIX; i++) {
			column = table.getColumnModel().getColumn(i);
			column.setPreferredWidth(20);
		}

		ListSelectionModel rowSM = table.getSelectionModel();
		rowSM.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				//Ignore extra messages.
				if (e.getValueIsAdjusting()) return;
       	          
				ListSelectionModel sm = (ListSelectionModel)e.getSource();
				if (!sm.isSelectionEmpty()) {
					selectedRow = sm.getMinSelectionIndex();
				}
				textCode.setText(getHexStr(getCurrCode()));
			}
		});

		ListSelectionModel colSM = table.getColumnModel().getSelectionModel();
		colSM.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				//Ignore extra messages.
				if (e.getValueIsAdjusting()) return;
       	          
				ListSelectionModel sm = (ListSelectionModel)e.getSource();
				if (!sm.isSelectionEmpty()) {
					selectedCol = sm.getMinSelectionIndex();
				}
				textCode.setText(getHexStr(getCurrCode()));
			}
		});

		repaint();
		return table;
	}

	public String getHexStr(int val) {
		String tmpStr = "0000" + Integer.toHexString(val);
		return (tmpStr.substring(tmpStr.length()-4, tmpStr.length()));
	}

	public int getCurrCode() {
		int codeVal = initVal + selectedRow*RADIX + selectedCol;
		return codeVal;
	}

	public Character getCurrChar() {
		int codeVal = initVal + selectedRow*RADIX + selectedCol;
		return (new Character((char)(codeVal)));
	}

	public void insertCurrCharacter() {
		String currChar;
		try {
			int code = Integer.parseInt(textCode.getText(), RADIX);
			currChar = (new Character((char)code)).toString();
		} catch (NumberFormatException nfe) {
			currChar = getCurrChar().toString();
			textCode.setText(getHexStr(getCurrCode()));
				textCode.setText(getHexStr(getCurrCode()));
		}

		int pos = textString.getCaretPosition();
		String newStr = textString.getText().substring(0, pos);
		newStr += currChar;
		newStr += textString.getText().substring(pos);
		textString.setText(newStr);
		textString.setCaretPosition(pos+1);
	}

	public boolean changePage(int code) {
		if (code < 0 || code > 0xFFFF) return false;

		initVal = code;
		textCode.setText(getHexStr(getCurrCode()));
		textPage.setText(getHexStr(initVal));
		repaint();

		return true;
	}

	public void pageUp() {
		changePage(initVal + RADIX*RADIX);
	}

	public void pageDown() {
		changePage(initVal - RADIX*RADIX);
	}

	class ButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JComponent c = (JComponent) e.getSource();

				if(c == btnSelect) {
					insertCurrCharacter();
				} else if(c == btnInsert) {
					int pos = myArea.getCaretPosition();
					myArea.insert(textString.getText(), pos);
				} else if(c == btnClear) {
					textString.setText("");
				} else if(c == btnClose) {
					dispose();
				} else if(c == btnUp) {
					pageUp();
				} else if(c == btnDown) {
					pageDown();
				} else if(c == btnGo) {
					try {
						int code = Integer.parseInt(textPage.getText(), RADIX);
						if (!changePage(code)) {
							textPage.setText(getHexStr(initVal));
						}
					} catch (NumberFormatException nfe) {
						textPage.setText(getHexStr(initVal));
					}
				}
		}
	}

	class ComboListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			JComponent c = (JComponent) e.getSource();

			if(c == cmbLang) {
				String lang = (String) e.getItem();
				if (lang.equals("English")) {
					changePage(0);
				} else if (lang.equals("Japanese")) {
					changePage(0x3040);
				} else if (lang.equals("Korean")) {
					changePage(0xAC00);
				} else if (lang.equals("Russian")) {
					changePage(0x0400);
				}
			}
		}
	}


}
