package did.xml;

import java.util.Vector;
import java.util.Enumeration;

import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Attr;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.DOMException;
import org.w3c.dom.NodeList;

/*******************************************************************************
*<pre>
*File:         DIDEntityType.java
*Version:      1.0
*Description:  
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
*******************************************************************************/

public class DIDEntityType {
	public static final int UNDETERMINED = -2;
	public static final int UNSPECIFIED = -1;
	public static final int PAGE = 0;
	public static final String PAGENAME = "Page";
	public static final String OTHER = "Other";
	private static final int builtInNum = 4;
	public String BuiltInEntityTypeNames[] = {"Zone", "Line", "Word", "Character"};
	public int BuiltInEntityTypes[] = { 1, 2, 3, 4 };
	public int BuiltInEntityLevels[] = { 1, 2, 3, 4 };
	public String EntityTypeNames[] = {"Zone", "Line", "Word", "Character"};
	public int EntityTypes[] = { 1, 2, 3, 4 };
	public int EntityLevels[] = { 1, 2, 3, 4 };
	protected int length = 4;
	private boolean entityExist = false;

//DEL DOCUMENT = "Document", PAGE = "Page", ZONE = "Zone", WORD = "Word", LINE = "Line", CHARACTER = "Character", VERTEX = "Vertex", OTHER = "Other";

	/**********************************************************************
	* Method used to instantiate this class.
	**********************************************************************/
	public DIDEntityType() {
	}

	/***********************************************************************
	* Remove all types.
	* @param
	* @return
	***********************************************************************/
	public void clearAll() {
		EntityTypeNames = null;
		EntityTypes = null;
		length = 0;
	}

	/***********************************************************************
	* Restore built-in types.
	* @param
	* @return
	***********************************************************************/
	public void restoreBuiltIn() {
		for (int i=0; i < builtInNum; i++) {
			add(BuiltInEntityTypeNames[i]);
		}
	}


	/***********************************************************************
	* 
	* @param
	* @return if the type name is not found, return null.
	***********************************************************************/
	public void add(String name) {
		String newNames[] = new String[length+1];
		int newTypes[] = new int[length+1];
		int newLevels[] = new int[length+1];
		int maxType = 0;

		for (int i=0; i < length; i++) {
			if (EntityTypeNames[i].equalsIgnoreCase(name)) {
				return;
			} else {
				newNames[i] = EntityTypeNames[i];
				newTypes[i] = EntityTypes[i];
				newLevels[i] = EntityLevels[i];
				if (EntityTypes[i] > maxType) {
					maxType = EntityTypes[i];
				}
			}
		}

		newNames[length] = name;
		newTypes[length] = maxType + 1;
		newLevels[length] = maxType + 1;
		length++;
		EntityTypeNames = newNames;
		EntityTypes = newTypes;
		EntityLevels = newLevels;
	}

	/***********************************************************************
	* 
	* @param
	* @return if the type name is not found, return null.
	***********************************************************************/
	public void add(String name, int level) {
		String newNames[] = new String[length+1];
		int newTypes[] = new int[length+1];
		int newLevels[] = new int[length+1];
		int maxType = 0;
		int pos = 0;

		for (int i=0; i < length; i++) {
			if (EntityTypeNames[i].equalsIgnoreCase(name)) {
				return;
			} else if (EntityLevels[i] <= level) {
				pos = i + 1;
				if (EntityTypes[i] > maxType) {
					maxType = EntityTypes[i];
				}
			}
		}

		length++;
		for (int i=0; i < pos; i++) {
			newNames[i] = EntityTypeNames[i];
			newTypes[i] = EntityTypes[i];
			newLevels[i] = EntityLevels[i];
		}
		newNames[pos] = name;
		newTypes[pos] = maxType + 1;
		newLevels[pos] = level;
		for (int i=pos+1; i < length; i++) {
			newNames[i] = EntityTypeNames[i-1];
			newTypes[i] = EntityTypes[i-1] + 1;
			newLevels[i] = EntityLevels[i-1];
		}
		EntityTypeNames = newNames;
		EntityTypes = newTypes;
		EntityLevels = newLevels;
	}

	/***********************************************************************
	* Method to get the name of entity type from the index of the array
	* storing entity type names.
	* @param
	* @return
	***********************************************************************/
	public String getName(int index) { 
		if (index >= 0 && index < length) {
			return (EntityTypeNames[index]);
		} else {
			return null;
		}
	}

	/***********************************************************************
	* 
	* @param
	* @return if the type name is not found, return null.
	***********************************************************************/
	public String typeToName(int num) { 
		for (int i=0; i < length; i++)  {
			if (EntityTypes[i] == num) {
				return (EntityTypeNames[i]);
			}
		}

		return null;
	}

	/***********************************************************************
	* Method to get type number from type name
	* @param
	* @return if the type name is not found, return UNDETERMINED.
	***********************************************************************/
	public int nameToType(String name) { 
		for (int i=0; i < length; i++)  {
			if (EntityTypeNames[i].equalsIgnoreCase(name)) {
				return (EntityTypes[i]);
			}
		}

		return UNDETERMINED;
	}

	/***********************************************************************
	* Method to get type number from index
	* @param
	* @return if the type name is not found, return UNDETERMINED.
	***********************************************************************/
	public int getType(int index) { 
		if (index >= 0 && index < length) {
			return (EntityTypes[index]);
		}
		else {
			return UNDETERMINED;
		}
	}

	/***********************************************************************
	* Method to get index from type name
	* @param
	* @return if the type name is not found, return UNDETERMINED.
	***********************************************************************/
	public int getIndex(String name) { 
		for (int i=0; i < length; i++)  {
			if (EntityTypeNames[i].equalsIgnoreCase(name)) {
				return i;
			}
		}

		return UNDETERMINED;
	}

	/***********************************************************************
	* Method to get index from type num
	* @param
	* @return if the type name is not found, return UNDETERMINED.
	***********************************************************************/
	public int getIndex(int num) { 
		for (int i=0; i < length; i++)  {
			if (EntityTypes[i] == num) {
				return i;
			}
		}

		return UNDETERMINED;
	}

	/***********************************************************************
	* 
	* @param
	* @return
	***********************************************************************/
	public Enumeration getTypeNames() { 
		Vector v = new Vector();

		for (int i=0; i < length; i++)  {
			v.add(EntityTypeNames[i]);
		}

		return (v.elements());
	}

	/***********************************************************************
	* 
	* @param
	* @return
	***********************************************************************/
	public Enumeration getBuiltInTypeNames() { 
		Vector v = new Vector();

		for (int i=0; i < builtInNum; i++)  {
			v.add(EntityTypeNames[i]);
		}

		return (v.elements());
	}

	/***********************************************************************
	* 
	* @param
	* @return
	***********************************************************************/
	public boolean isValid(String name) { 
		for (int i=0; i < length; i++)  {
			if (EntityTypeNames[i].equalsIgnoreCase(name)) {
				return true;
			}
		}

		return false;
	}

	/***********************************************************************
	* 
	* @param
	* @return
	***********************************************************************/
	public boolean ancestor(int type1, int type2) { 
		if (getIndex(type1) < getIndex(type2)) {
			return true;
		} else {
			return false;
		}
	}

	/***********************************************************************
	* 
	* @param
	* @return
	***********************************************************************/
	public int length() { 
		return (length);
	}


	/***********************************************************************
	* Read types from the XML structure.
	* @param
	* @return
	***********************************************************************/
	public void buildTypes(Document xmlRoot) {
		NodeList myNodeList = xmlRoot.getChildNodes();
		int myLength = (myNodeList != null) ? myNodeList.getLength() : 0;

		entityExist = false;
		/*DEL*/System.out.println(">>Begin extract entity structure");
		for (int i = 0; i < myLength; i++) {
			Node tmpNode = myNodeList.item(i);
			processNode(tmpNode, 1);
		}
		if (!entityExist) {
			restoreBuiltIn();
		}
		/*DEL*/System.out.println("**End extract entity structure");
	}

	/***********************************************************************
	* 
	* @param
	* @return
	***********************************************************************/
        private void processNode(Node node, int level) {
		int newlevel = level;

		if (node.getNodeType() == Node.ELEMENT_NODE) {
			NamedNodeMap attrs = node.getAttributes();
			Node entityAttr = attrs.getNamedItem("Entity");
			String isEntity = null;
			if (entityAttr != null) {
				isEntity=entityAttr.getNodeValue();
			}
			if(((String)"True").equalsIgnoreCase(isEntity)) {
				newlevel++;
				add(node.getNodeName(), level);
				entityExist = true;
			}
			NodeList myNList = node.getChildNodes();
			int len = (myNList != null) ? myNList.getLength() : 0;
			for (int i = 0; i < len; i++) {
				Node tmpNode = myNList.item(i);
				if (tmpNode.getNodeType() != Node.TEXT_NODE) {
					processNode(tmpNode, newlevel);
				}
			}
		}
        }

}

