package did.xml;

import java.awt.Component;
import java.util.Enumeration;

import javax.swing.JTree;
import javax.swing.ImageIcon;
import javax.swing.tree.DefaultTreeCellRenderer;

import org.w3c.dom.*;

import did.xml.DIDNode;
import did.xml.DIDDefaultTreeModel;

/*****************************************************************************************
*<pre>
*File:         DIDTree.java
*Version:      1.1
*Description:  This class is responsible for custom rendering each cell in the DIDTree
*it is registerd to. Custom rendering involves setting the icon of each cell to a specific
*image based on the type of data the node represents.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
******************************************************************************************/

public class DIDTreeCellRenderer extends DefaultTreeCellRenderer {
	/** Image for element nodes. */
	ImageIcon elementIcon	= new ImageIcon("pics//common.jpg");
	/** Image for leaf nodes. */
	ImageIcon leafIcon		= new ImageIcon("pics//attribute.jpg");
	/** Image for page nodes. */
	ImageIcon pageIcon		= new ImageIcon("pics//page.jpg");
	/** Image for zone nodes. */
	ImageIcon zoneIcon		= new ImageIcon("pics//zone.jpg");
	/** Image for line nodes. */
	ImageIcon lineIcon		= new ImageIcon("pics//line.jpg");
	/** Image for line nodes. */
	ImageIcon wordIcon		= new ImageIcon("pics//word.jpg");
	/** Image for character nodes. */
	ImageIcon characterIcon	= new ImageIcon("pics//character.jpg");

	DIDDefaultTreeModel myModel = null;

	/**********************************************************************
	* Method used to instantiate the class.
	* @param myModel A reference to the TreeModel providing the data
	**********************************************************************/
	public DIDTreeCellRenderer(DIDDefaultTreeModel model) {
		super();
		myModel = model;
	}


	/*****************************************************************************************
	* Method used to return a component for custom redering of cells is the DIDTree.  Icon 
	* based on type of data node contains.
	* @param tree is the JTree the receiver is being configured for.
	* @param value Value of the current tree cell.
	* @param sel True, cell drawn in selected state.
	* @param expanded True, cell drawn in expanded state.
	* @param leaf True, cell drawn in leaf state.
	* @param row Integer denoting the row this cell occupies.
	* @param hasFocus True if the node currently has focus. 
	* @return Component that the renderer uses to draw the value.
	*****************************************************************************************/   
	public Component getTreeCellRendererComponent(JTree tree, Object value,
			boolean sel, boolean expanded, boolean leaf,
			int row, boolean hasFocus) {
		setLeafIcon(leafIcon);
		if ((value != null) && (value instanceof DIDNode)) {
			DIDNode myDIDNode = (DIDNode)value;
			Node node = myDIDNode.getDomNode();
			if (node.getNodeName().startsWith(DIDNode.PAGE)) {
				setClosedIcon(pageIcon);
				setOpenIcon(pageIcon);
				setToolTipText("This is a Page element");
			} else if (node.getNodeName().startsWith(DIDNode.ZONE)) {
				setClosedIcon(zoneIcon);
				setOpenIcon(zoneIcon);
				setToolTipText("This is a Zone element");
			} else if (node.getNodeName().startsWith(DIDNode.LINE)) {
				setClosedIcon(lineIcon);
				setOpenIcon(lineIcon);
				setToolTipText("This is a Line element");
			} else if (node.getNodeName().startsWith(DIDNode.WORD)) {
				setClosedIcon(wordIcon);
				setOpenIcon(wordIcon);
				setToolTipText("This is a Word element");
			} else if (node.getNodeName().startsWith(DIDNode.CHARACTER)) {
				setClosedIcon(characterIcon);
				setOpenIcon(characterIcon);
				setToolTipText("This is a Character element");
			} else {
				boolean isvalid = false;

				Enumeration names = myModel.EntityType.getTypeNames();
				while (names.hasMoreElements()) {
					String name = (String)names.nextElement();
					if (node.getNodeName().startsWith(name)) {
						isvalid = true;
						break;
					}
				}
				if (isvalid) {
					setClosedIcon(zoneIcon);
					setOpenIcon(zoneIcon);
					setToolTipText("This is an entity");
				} else {
					setClosedIcon(elementIcon);
					setOpenIcon(elementIcon);
					setToolTipText("This is some other element");
				}
			}
        }
	return(super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus));
    }
}
