package did.xml;

import java.io.File;
import java.io.FileWriter;
import java.io.FileInputStream;

import java.lang.Boolean;

import java.util.Vector;
import java.util.Enumeration;

import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;

import org.xml.sax.InputSource;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Attr;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.DOMException;
                          
import com.ibm.xml.dom.ElementImpl;
import com.ibm.xml.dom.DocumentImpl;
import com.ibm.xml.parsers.DOMParser;

import did.io.DOMWriter;
import did.xml.DIDNode;
import did.xml.DIDEntityType;
import did.gui.DIDBaseRoi;
import did.gui.DIDRoiIFC;
import did.gui.DIDPolyRoi;
import did.gui.DIDProperties;

/*****************************************************************************************
*<pre>
*File:         DIDDefaultTreeModel.java
*Version:      1.0
*Description:  Implementation of the Java DefaultTreeModel.  A DefaultTreeModel handles 
*data stored in a series of nodes organized in a tree structure.  An XML document can be 
*parsed into a DOM tree which is also a series of nodes organized into a tree structure 
*but Java can not use the DOM tree directly.  So, to accomodate this shortcoming a Java
*based node structure (DIDNodes) is create based off of the DOM Tree.  The Java tree 
*references the corresponding node in the DOM Tree by setting the DOM node as its 
*userObject.  This way the Java Tree is always aware of what part of the DOM tree it 
*represents.
*NOTE: This class also contains several attributes that also make reference to the 
*Document node of independed DOM trees.  Their purpose is aid in the creation of new data
*files (Pages) as well as aid in the creation of specific portions or the data file 
*architecture (Zone, Line, Charater, Vertex).
*This architecture is a stopgap messure until an adeqaute and efficient means of generating
*well formed, valid XML directly from the DTD can be established.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
*@see DIDNode
******************************************************************************************/

public class DIDDefaultTreeModel extends DefaultTreeModel {

	/** A String object used to hold any error message generated during the parsing of the XML file.*/
	private String					myParseError	= null;
	/** Attribute the points to the parser object responsible for parsing raw XML into a DOM tree. */
	private DOMParser				myParser		= new DOMParser();
	/** A reference to a current XML Document being edited.  More specifically this attribute references
	the first node in the DOM tree which is the document node.*/
	private Document				myDocument		= null;	
	/** Attribute that holds the DOM representation of the protion of XML which contains all the elements
	and attributes for a Page.*/
	private Document				myPageStub		= null;
	/** Attribute that holds the DOM representation of the protion of XML which contains all the elements
	and attributes for a Zone.*/
	private Document				myZoneStub		= null;
	/** Attribute that holds the DOM representation of the protion of XML which contains all the elements
	and attributes for a Line.*/
	private Document				myLineStub		= null;
	/** Attribute that holds the DOM representation of the protion of XML which contains all the elements
	and attributes for a Word.*/
	private Document				myWordStub		= null;
	/** Attribute that holds the DOM representation of the protion of XML which contains all the elements
	and attributes for a Character.*/
	private Document				myCharacterStub	= null;
	/** Attribute that holds the DOM representation of the protion of XML which contains all the elements
	and attributes for a Vertex.*/
	private Document				myVertexStub	= null;
	/** Attribute that specify the entity types */
	public static DIDEntityType EntityType = new DIDEntityType();
	/** Attribute that holds a reference to the class the listens to the events of this model. */
	private DIDTreeModelListener	myListener		= new DIDTreeModelListener();
	/** These constants define the behavior of myROIs.  INIT_CAPACITY sets the initial
	capacity of the vector while INCREMENT_NUM sets the amount by which the capacity is 
	increased when the vector overflows.*/
	private final static int	INIT_CAPACITY	= 23,
								INCREMENT_NUM	= 3;
	/** Vector whose purpose is to hold and track all the DIDRoiIFC object implementations
	currently on the screen. */
	public static Vector		myROIs			= new Vector(INIT_CAPACITY, INCREMENT_NUM);

	/*****************************************************************************************
	* Method used to instantiate this class.  
	* @param root The root node for this models node tree. 
	*****************************************************************************************/   
	public DIDDefaultTreeModel(TreeNode root) {
		super(root);
		init();
	}
          
	/*****************************************************************************************
	* Method used to instantiate this class.  This constructor creates a node based on the 
	* DIDNode structure (DOM node as userObject) and sets it as the root node for this model. 
	* Basically this is used to initialize the model.
	* @param root The root node for this models node tree. 
	* @param nodeMessage The name of the node created by this method.
	*****************************************************************************************/   
	public DIDDefaultTreeModel(TreeNode root, String nodeMessage) {
		super(root);
		DocumentImpl tmpDoc = new DocumentImpl();
		ElementImpl tmpNode = new ElementImpl(tmpDoc, nodeMessage);
		setRoot(new DIDNode(tmpNode, this));
		init();
	}

	/*****************************************************************************************
	* Method used to initialize this class.  In this implmentation all the data stubs are 
	* loaded into the appropriate attributes.  The stubs serve as well formed canned peices 
	* of XML specific to this application.  
	* @see #loadStub
	*****************************************************************************************/   
	private void init() {
		addTreeModelListener(new DIDTreeModelListener());
		loadStub(DIDNode.VERTEX);
		loadStub(DIDNode.CHARACTER);
		loadStub(DIDNode.WORD);
		loadStub(DIDNode.LINE);
		loadStub(DIDNode.ZONE);
		loadStub(DIDNode.PAGE);
	}

	/*****************************************************************************************
	* Method used to respond to the editing of a node in the tree.  When editing of a node is 
	* completed the DIDTreeCellEditor calls this method passing in the path to the node edited
	* and the new value for the node.  This implementation takes that value and assigns it to
	* the DOM node the node at the end of the path is mirroring.
	* @param path TreePath to the node being edited.
	* @param newValue new value for the node being edited.
	*****************************************************************************************/   
	public void valueForPathChanged(TreePath path, Object newValue){
		DIDNode myDIDNode = (DIDNode)path.getLastPathComponent();
		String nodeValue = (String)newValue;
		myDIDNode.setNodeValue(nodeValue);
	}

	/*****************************************************************************************
	* Method used to build the tree of nodes this model manages. 
	* @param fileName Fully qualified path to the XML file this model should mirror.
	*****************************************************************************************/   
	public void buildDIDModel(String fileName) {	
		if (getDocument(fileName)) {
			DIDNode myRootNode = new DIDNode(myDocument, this);
			NodeList myNodeList = myDocument.getChildNodes();
			int myLength = (myNodeList != null) ? myNodeList.getLength() : 0;
			/*DEL*/System.out.println(">>Begin processNode");
			for (int i = 0; i < myLength; i++) {
				Node tmpNode = myNodeList.item(i);
				processNode(tmpNode, myRootNode);
			}
			/*DEL*/System.out.println("**End processNode");
			setRoot(myRootNode);
			/*DEL*/System.out.println(">>Begin Create Rois");
			/// Create ROIs
			for (Enumeration nodes = findNodesHaving(null, "Corners"); nodes.hasMoreElements(); ) { 
				DIDNode tmpZone = (DIDNode)nodes.nextElement();
				buildROI(tmpZone, tmpZone.getNodeName());
			}
			/*DEL*/System.out.println("**End Create Rois");
/*DEL->
			/// Create Zone ROIs //
			for (Enumeration zones = findNodes(null, DIDNode.ZONE); zones.hasMoreElements(); ) { 
				DIDNode tmpZone = (DIDNode)zones.nextElement();
				buildROI(tmpZone, DIDNode.ZONE);
			}
			/// Create Line ROIs //
			for (Enumeration lines = findNodes(null, DIDNode.LINE); lines.hasMoreElements() ;) { 
				DIDNode tmpLine = (DIDNode)lines.nextElement();
				buildROI(tmpLine, DIDNode.LINE);
			}
			/// Create Word ROIs //
			for (Enumeration words = findNodes(null, DIDNode.WORD); words.hasMoreElements() ;) { 
				DIDNode tmpWord = (DIDNode)words.nextElement();
				buildROI(tmpWord, DIDNode.WORD);
			}
			/// Create Character ROIs //
			for (Enumeration chars = findNodes(null, DIDNode.CHARACTER); chars.hasMoreElements() ;) { 
				DIDNode tmpChar = (DIDNode)chars.nextElement();
				buildROI(tmpChar, DIDNode.CHARACTER);
			}
//<-DEL*/
			/*DEL*/System.out.println(">>Begin build logical links");
			buildLogicalLinks(myRootNode);
			/*DEL*/System.out.println("**End build logical links");
		} else {
			DefaultMutableTreeNode myErrorRoot = new DefaultMutableTreeNode("An error occured while parsing the XML.");
			myErrorRoot.add(new DefaultMutableTreeNode(myParseError));
			setRoot(myErrorRoot);
		}
	}

	/****************************************************************************************
	* This function is responsible for parsing an XML file.  It creates a FileInputStream 
	* from the fileName passed to it as a parameter.  This stream is then passed to a parser
	* objecr reference (myParser) which then turns the raw XML into a DOM tree.  The attribute
	* myDocument is then set to the document node (root) of the successfully parsed XML file.
	* @param fileName A string that is the fully qualified path to the XML file to be parsed.
	* @return This function returns TRUE if the parsing was successful and FALSE if some 
	* error occurs.
	*****************************************************************************************/
	private boolean getDocument(String fileName) {
		try	{
			FileInputStream myInputStream = new FileInputStream(fileName);
			InputSource	mySource = new InputSource(myInputStream);
			myParser.setNodeExpansion(DOMParser.FULL);
			myParser.setContinueAfterFatalError(true);
			myParser.setAllowJavaEncodingName(true);
			myParser.parse(mySource);
			myDocument = myParser.getDocument();
			myInputStream.close();
        } catch (Exception e) {
            e.printStackTrace(System.err);
			myParseError = e.toString();
			return(false);
        }
		return(true);
	}

	/*****************************************************************************************
	* Method used to recursively walk down a DOM tree creating DIDNode for each DOM node thus 
	* creating a corresponding Java TreeNode structure that precisely mimics the DOM tree. 
	* @param node W3C node being processed.
	* @param parent Reference to the parent node of the node being processed.
	*****************************************************************************************/   
	private void processNode(Node node, DIDNode parent) {
		if (node.getNodeType() == Node.ELEMENT_NODE) {
			DIDNode child = new DIDNode(node, this);
			insertNode(child, parent);
			NamedNodeMap attrs = node.getAttributes();
			int attrCount = (attrs != null) ? attrs.getLength() : 0;
			for (int i = 0; i < attrCount; i++) {
				Node attr = attrs.item(i);
				insertNode(new DIDNode(attr, this), child);
			}
			NodeList myNList = node.getChildNodes();
			int len = (myNList != null) ? myNList.getLength() : 0;
			for (int i = 0; i < len; i++) {
				Node tmpNode = myNList.item(i);
				if (tmpNode.getNodeType() != Node.TEXT_NODE) {
					processNode(tmpNode, child);
				}
			}
		}
	}


	/*****************************************************************************************
	* Method used to safely insert a node into this model's tree structure.
	* @param newChild The child node to be inserted.
	* @param parent The parent of the node to be inserted.
	*****************************************************************************************/   
	private void insertNode(DIDNode newChild, DIDNode parent){ 
		insertNodeInto(newChild, parent, parent.getChildCount());
	}	

	/*****************************************************************************************
	* Method used to persist the DOM tree structure rooted at myDocument to a physical storage
	* device. 
	* @param fileName Fully qualified path to the file that the DOM tree is to be saved.
	*****************************************************************************************/   
	public void saveModel(String fileName) {
		try {
			DOMWriter myWriter = new DOMWriter(fileName);
			DIDNode tmpDIDNode = (DIDNode)getRoot();
			Node tmpNode = tmpDIDNode.getDomNode();
			myWriter.print(tmpNode);
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
	}

	/*****************************************************************************************
	* Convenience method used to print out the structure of the nodes rooted at the startNode.
	* @param startNode Node from which to begin recursively walking down.
	*****************************************************************************************/   
	public void printTree(DIDNode startNode) {
		try {
			for (Enumeration e = startNode.preorderEnumeration() ; e.hasMoreElements() ;) { 
				DIDNode tmpNode = (DIDNode)e.nextElement();
				System.out.println(tmpNode.toString());
			}
		} catch (Exception e) { }
	}

	/*****************************************************************************************
	* Method used to return all the ROIs built from the open docuement (XML file loaded into
	* the myDocument attibute).
	* @return A vector containing all ROIs.
	*****************************************************************************************/   
	public Vector getROIs() { return (myROIs); }

	/*****************************************************************************************
	* Method used to build ROI objects from the XML.  This method walks down the node tree 
	* starting at the anchorNode and finds any node whose name begins with "Vertex" and whose
	* parent node's name begins with roiType and ends with "Corners".  This node's children 
	* contains vertex information that is placed in a Vector based on whether it is an x or 
	* y coordinate. These Vectors are then used to construct a new ROI.
	* @param anchorNode A DIDNode representing the starting point from which Verticies (a set
	* of x, y corrdinates) can be extracted.
	* @param roiType A String denoting the type of data (Zone, Line, Character)
	*****************************************************************************************/   
	public void buildROI(DIDNode anchorNode, String roiType) {
		Vector x = new Vector();
		Vector y = new Vector();
		int i = 0;
		/** Loop through the data model starting at the node passed in as a parameter(anchornode) and
		look for those nodes whose value is equal to the roiType*/
		for (Enumeration element = anchorNode.breadthFirstEnumeration() ; element.hasMoreElements() ;) { 
			DIDNode tmpElement = (DIDNode)element.nextElement();
			/** Identify those nodes that contain Vertex data (i.e. any node that ends in Corner with
			the proper prefix passed into this method as a parameter.*/
			DIDNode parent = (DIDNode)tmpElement.getParent();
			if (tmpElement.getNodeName().equals("Vertex") && parent.getNodeName().equals(roiType + "Corners")) {
				/** Once the proper node has been identified, loop through its children.  There should 
				be two children; one containing x coordinates and one containing y coordinates.  Add the 
				values of exh node to the correct Vector. */
				for (Enumeration attribute = tmpElement.children() ; attribute.hasMoreElements() ;) {
					DIDNode tmpAttribute = (DIDNode)attribute.nextElement();
					if (tmpAttribute.getNodeName().equals("x")) {
						x.add(new Integer(tmpAttribute.getNodeValue()));
					} else if (tmpAttribute.getNodeName().equals("y")) {
						y.add(new Integer(tmpAttribute.getNodeValue()));
					}
				}
				i++;
			}
		}
		/** Now build an RIO from all the verticies that have been extracted.  A DIDPolyRoi object is used
		to simplify the task.  That way the number of verticies and their arrangement are unimportant.*/
		DIDPolyRoi tmpRoi = new DIDPolyRoi(vectorToInt(x), vectorToInt(y), i);
		DIDProperties newProps = DIDProperties.getInstance();
		Boolean showCoords = new Boolean(newProps.getProperty(roiType + "Coordinates"));
		tmpRoi.setCoordDisp(showCoords.booleanValue());
		tmpRoi.setAnchorNode(anchorNode, getAttribute(anchorNode, "Color"));
		tmpRoi.setStatus(DIDRoiIFC.NORMAL);
		anchorNode.setRoi(tmpRoi); 
		myROIs.add(tmpRoi);
	}

	/***********************************************************************
	* Method used to build logical links.  This method walks down the node 
	* tree starting at the anchorNode and if any node have a child node
	* name ends with "Next", make a link between the node and the next node
	* whose ID is the value of "Next" node.
	* @param anchorNode is a starting point.
	***********************************************************************/
	public void buildLogicalLinks(DIDNode anchorNode) {
		Enumeration nodes = findNodesHaving(null, "Next");
		while (nodes.hasMoreElements()) { 
			DIDNode tmpNode = (DIDNode)nodes.nextElement();
			String nextID = getAttribute(tmpNode, tmpNode.getNodeName()+"Next");
			DIDNode next = getNodeByID((DIDNode)tmpNode.getParent(), nextID);
			if (next != null && !nextID.equals("nil") && !nextID.equals(""))  {
				if(tmpNode.getRoi() != null && next.getRoi() != null) {
					tmpNode.getRoi().setNextRoi((DIDBaseRoi)next.getRoi());
					next.getRoi().setPrevRoi((DIDBaseRoi)tmpNode.getRoi());
				}
			}
		}
	}

	/*****************************************************************************************
	* Method used to locate a specific set of nodes by the name their userObjects (DOM Nodes).
	* @param nodeName A String that is the name of the node this method should find.
	* @return An Enumeration of DIDNodes whose userObject's names matche the nodeName 
	* parameter.
	*****************************************************************************************/   
	public Enumeration findNodes(DIDNode anchorNode, String nodeName) {
		Vector nodeVector = new Vector();
		if (anchorNode == null) { anchorNode = (DIDNode)getRoot(); }
		for (Enumeration e = anchorNode.breadthFirstEnumeration(); e.hasMoreElements() ;) { 
			DIDNode tmpNode = (DIDNode)e.nextElement();
			if (tmpNode.getNodeName().equals(nodeName)) {
				nodeVector.add(tmpNode);
			}
		}
		return nodeVector.elements();
	}

	/***********************************************************************
	* Method used to find a specific set of nodes the suffix of whose name
	* equals to the suffix parameter.
	* @param suffix A String that is the suffix of the node name this method
	* should find.
	* @return An Enumeration of DIDNodes whose userObject's name ends with
	* the suffix parameter.
	***********************************************************************/
	public Enumeration findNodesHaving(DIDNode anchorNode, String suffix) {
		Vector nodeVector = new Vector();
		if (anchorNode == null) { anchorNode = (DIDNode)getRoot(); }
		for (Enumeration e = anchorNode.breadthFirstEnumeration(); e.hasMoreElements() ;) { 
			DIDNode tmpNode = (DIDNode)e.nextElement();
			if (tmpNode.getNodeName().endsWith(suffix)) {
				DIDNode parent = (DIDNode)tmpNode.getParent();
				if(parent != null && tmpNode.getNodeName().equals(parent.getNodeName()+suffix)) {
					nodeVector.add(parent);
				}
			}
			DIDNode parent = (DIDNode)tmpNode.getParent();
			if(parent != null && tmpNode.getNodeName().equals(parent.getNodeName()+"Attributes")) {
				Enumeration e2 = findChildren(tmpNode, parent.getNodeName()+suffix);
				if(e2.hasMoreElements()) {
					nodeVector.add(parent);
				}
			}
		}
		return nodeVector.elements();
	}

	/***********************************************************************
	* Method used to get the first node whose ID is same as the idName
	* parameter.
	* @param idName A String that is the ID of the node this method should
	* find.
	* @return The DIDNodes whose userObject's ID matches the idName
	* parameter.
	***********************************************************************/
	public DIDNode getNodeByID(DIDNode anchorNode, String idName) {
		Vector nodeVector = new Vector();
		if (anchorNode == null) { anchorNode = (DIDNode)getRoot(); }
		Enumeration e = anchorNode.breadthFirstEnumeration();
		while (e.hasMoreElements()) { 
			DIDNode tmpNode = (DIDNode)e.nextElement();
			String nodeID = getAttribute(tmpNode,
						tmpNode.getNodeName()+"ID");
			if(nodeID != null && nodeID.equals(idName)) {
				return tmpNode;
			}
		}
		return null;
	}

	/***********************************************************************
	* Method used to locate a specific set of nodes by the ID.
	* @param idName A String that is the ID of the node this method should
	* find.
	* @return An Enumeration of DIDNodes whose userObject's IDs match
	* the idName parameter.
	***********************************************************************/
	public Enumeration findNodesByID(DIDNode anchorNode, String idName) {
		Vector nodeVector = new Vector();
		if (anchorNode == null) { anchorNode = (DIDNode)getRoot(); }
		Enumeration e = anchorNode.breadthFirstEnumeration();
		while (e.hasMoreElements()) { 
			DIDNode tmpNode = (DIDNode)e.nextElement();
			String nodeID = getAttribute(tmpNode,
						tmpNode.getNodeName()+"ID");
			if(nodeID != null && nodeID.equals(idName)) {
				nodeVector.add(tmpNode);
			}

		}
		return nodeVector.elements();
	}

	/***********************************************************************
	* Method used to get the first children of a specific node whose name
	* is same as the nodeName parameter.
	* @param nodeName A String that is the name of the node this method
	* should find.
	* @return the first DIDNode whose name matches the nodeName parameter.
	***********************************************************************/
	public DIDNode getChild(DIDNode anchorNode, String nodeName) {
		if (anchorNode == null) { anchorNode = (DIDNode)getRoot(); }
		Enumeration e = anchorNode.children();
		while (e.hasMoreElements()) { 
			DIDNode tmpNode = (DIDNode)e.nextElement();
			if (tmpNode.getNodeName().equals(nodeName)) {
				return tmpNode;
			}
		}
		return null;
	}

	/***********************************************************************
	* Method used to locate a specific set of children by the name.
	* @param nodeName A String that is the name of the node this method
	* should find.
	* @return An Enumeration of DIDNodes whose userObject's names matche
	* the nodeName parameter.
	***********************************************************************/
	public Enumeration findChildren(DIDNode anchorNode, String nodeName) {
		Vector nodeVector = new Vector();
		if (anchorNode == null) { anchorNode = (DIDNode)getRoot(); }
		for (Enumeration e = anchorNode.children(); e.hasMoreElements() ;) { 
			DIDNode tmpNode = (DIDNode)e.nextElement();
			if (tmpNode.getNodeName().equals(nodeName)) {
				nodeVector.add(tmpNode);
			}
		}
		return nodeVector.elements();
	}

	/***********************************************************************
	* Method used to find the value of entity attribute.
	* @param attrName is  a String that is the name of attribute to find.
	* @return a String which is the value of the attribute.
	* return null, if there is no such an attribute.
	***********************************************************************/
	public String getAttribute(DIDNode anchorNode, String attrName) {

		if (anchorNode == null) { anchorNode = (DIDNode)getRoot(); }
		String interNodeName = anchorNode.getNodeName()+"Attributes";

		Enumeration e = anchorNode.children();
		while (e.hasMoreElements()) { 
			DIDNode tmpNode = (DIDNode)e.nextElement();
			if (tmpNode.getNodeName().equals(interNodeName)) {
				DIDNode node = getChild(tmpNode, attrName);
				if(node != null) {
					DIDNode value = getChild(node, "Value");
					if(value != null) {
						return (value.getNodeValue());
					}
				}
			} else if (tmpNode.getNodeName().equals(attrName)) {
				DIDNode value = getChild(tmpNode, "Value");
				if(value != null) {
					return (value.getNodeValue());
				}
			}
		}

		return null;
	}


	/*****************************************************************************************
	* Method used to used to convert a Vector of integers into an array of integers.  In this 
	* implementation this method is called from the buildROI method.  The buildROI method 
	* passes in a Vector of ROI coordinates it has extracted from the data file.  The 
	* resulting arrays are used to construct and DIDRoiPoly object. 
	* @param inVector A java Vector object whose elements are integers representing a set of 
	* x or y coordinates.
	* @return An Integer array representing a set of x or y coordinates.
	*****************************************************************************************/   
	private int[] vectorToInt(Vector inVector) {
		inVector.trimToSize();
		int[] i = new int[inVector.size()];
		int ctr = 0;
		for (Enumeration e = inVector.elements(); e.hasMoreElements(); ) {
			Integer tmpInt = (Integer)e.nextElement();
			i[ctr] = tmpInt.intValue();
			ctr++;
		}
		return(i);
	}

	/*****************************************************************************************
	* Method used to create a document stub from an XML file. The stubs serve as well formed 
	* canned peices of XML specific to this application.  These stubs can then be processed and
	* placed into the Java and DOM trees whenever necessary.
	* @param dataType The type of data stub to load (Page, Zone, Line, Character).
	*****************************************************************************************/   
	public void loadStub(String dataType) {
		Document tmpDocument = null;
		try	{
			File myInputFile = new File(dataType + ".xml");
			FileInputStream myInputStream = new FileInputStream(myInputFile);
			InputSource	mySource = new InputSource(myInputStream);
			myParser.setNodeExpansion(DOMParser.FULL);
			myParser.setContinueAfterFatalError(true);
			myParser.setAllowJavaEncodingName(true);
			myParser.parse(mySource);
			tmpDocument = myParser.getDocument();
			myInputStream.close();
			if (dataType.equals(DIDNode.PAGE)) {
				myPageStub = tmpDocument;
				return;	
			} else if (dataType.equals(DIDNode.ZONE)) {
				myZoneStub = tmpDocument;
				return;	
			} else if (dataType.equals(DIDNode.LINE)) {
				myLineStub = tmpDocument;
				return;	
			} else if (dataType.equals(DIDNode.WORD)) {
				myWordStub = tmpDocument;
				return;	
			} else if (dataType.equals(DIDNode.CHARACTER)) {
				myCharacterStub = tmpDocument;
				return;
			} else if (dataType.equals(DIDNode.VERTEX)) {
				myVertexStub = tmpDocument;
				return;
			}
		} catch (Exception e) {
            e.printStackTrace(System.err);
			System.out.println("Error loading " + dataType + " XML stub.  Program cannot continue...");
			System.out.println("There seems to be a problem loading the peice of XML used to define how" +
				" to add ROIs(Zones, Line, etc.).  Please see your system administrator for assistance.");
			System.exit(0);
        }
	}
	
	/*****************************************************************************************
	* Method used to convert a DOM node from a stub into a valid portion of the active documents
	* structure.
	* NOTE: You CAN NOT directly insert part or all of one document into another document.  You
	* must use the documents create methods to add nodes (elements, attributes) to the document
	* in order to preserve document validity.
	* @param node The DOM node being processed.
	* @param DomParent The parent of the DOM node being processed.
	* @param JavaParent The Java parent node of the DOM node being processed.
	*****************************************************************************************/   
	private void processDomNode(Node node, Node DomParent, DIDNode JavaParent) {
		if (node.getNodeType() == Node.ELEMENT_NODE) {
			DIDNode elemChild = null;
			Element element = null;
			try {
				element = myDocument.createElement(node.getNodeName());
				Node tmpNode = DomParent.appendChild(element);
				elemChild = new DIDNode(tmpNode, this);
			} catch (DOMException e) {
				System.out.println("Exception in processDomNode() - Processing Element " + e);
			}
			if (elemChild != null) { insertNode(elemChild, JavaParent); }
			NamedNodeMap attrs = node.getAttributes();
            int attrCount = (attrs != null) ? attrs.getLength() : 0;
            for (int i = 0; i < attrCount; i++) {
                Node attr = attrs.item(i);
				DIDNode attrChild = null;
				try {
					Attr attribute = myDocument.createAttribute(attr.getNodeName());
					element.setAttributeNode(attribute);
					attrChild = new DIDNode(attribute, this);
				} catch (DOMException e) {
					System.out.println("Exception in processDomNode() - Processing Attribute " + e);
				}
				if (attrChild != null) { insertNode(attrChild, elemChild); }
			}
			NodeList nodeChildList = node.getChildNodes();
			int len = (nodeChildList != null) ? nodeChildList.getLength() : 0;
			for (int i = 0; i < len; i++) {
				Node tmpNode = nodeChildList.item(i);
				if (tmpNode.getNodeType() != Node.TEXT_NODE) {
					processDomNode(tmpNode, element, elemChild);
				}
			}
		}
	}

	/*****************************************************************************************
	* Method used to add the data behind an ROI (whether it be a Zone, Line, or Character) to 
	* both the Java and DOM Trees.  
	* @param n An Integer representing the number of Verticies this ROI contains.
	* @param anchorNode The parent node of the ROI being added. If null the Page assumed.
	* @param dataType A String representing the type of data the ROI represents (Zone, Line, etc.)
	* @return A DIDNode that is the root of the portion of the overall data tree that the 
	* ROI being added should set as its anchorNode.  
	* @see #processDomNode
	*****************************************************************************************/   
	public DIDNode addROI(int n, DIDNode anchorNode, String dataType) {
		if (anchorNode == null) { 
			for (Enumeration e = findNodes(null, DIDNode.PAGE); e.hasMoreElements() ;) { 
				anchorNode = (DIDNode)e.nextElement();
			}
		}
		/** Create a reference to the appropriate document stub. */
		Document docStub = null;
		if (dataType.equals(DIDNode.ZONE)) {
			docStub = myZoneStub;
		} else if (dataType.equals(DIDNode.LINE)) {
			docStub = myLineStub;
		} else if (dataType.equals(DIDNode.WORD)) {
			docStub = myWordStub;
		} else if (dataType.equals(DIDNode.CHARACTER)) {
			docStub = myCharacterStub;
		}
		NodeList myElementList = docStub.getChildNodes();
		int myLength = (myElementList != null) ? myElementList.getLength() : 0;
		for (int elemCtr = 0; elemCtr < myLength; elemCtr++) {
			Node tmpNode = myElementList.item(elemCtr);
			processDomNode(tmpNode, anchorNode.getDomNode(), anchorNode);
		}
		docStub = myVertexStub;
		anchorNode = (DIDNode)anchorNode.getChildAt(anchorNode.getChildCount() - 1);
		DIDNode roiNode = anchorNode;
		for (Enumeration e = findNodes(anchorNode, dataType + "Corners"); e.hasMoreElements(); ) {
			anchorNode = (DIDNode)e.nextElement();
			for (int vertCtr = 0; vertCtr < n; vertCtr++) {
				NodeList mySubElemList = docStub.getChildNodes();
				int mySubElemLength = (mySubElemList != null) ? mySubElemList.getLength() : 0;
				for (int nodeCtr = 0; nodeCtr < mySubElemLength; nodeCtr++) {
					Node tmpNode = mySubElemList.item(nodeCtr);
					processDomNode(tmpNode, anchorNode.getDomNode(), anchorNode);
				}
			}
		}
		return (roiNode);
	}
}
