package did.xml;

import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;

import did.xml.DIDTreeSelectionListener;
import did.xml.DIDDefaultTreeModel;
import did.xml.DIDTreeCellRenderer;
import did.xml.DIDTreeCellEditor;
import did.gui.DIDUnicodeEditor;

/*****************************************************************************************
*<pre>
*File:         DIDTree.java
*Version:      1.1
*Description:  This class is an extension of the Swing JTree class.  It acts a viewer for
*the TreeModel (XML) displaying the data as an outline.
*
*Modification Log:
*Date          Name    Comment
*xx-xx-xxxx    XXX	   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
*</pre>
******************************************************************************************/

public class DIDTree extends JTree{

	/** Attribute that holds a refernce to the TreeModel providing the data for this viewer. */
	DIDDefaultTreeModel	myModel		= null;
	/** Attribute the holds a reference to the TreeSelectionListerner responsible for detecting
	and responding to Tree selection events.*/
	DIDTreeSelectionListener mySListener = new DIDTreeSelectionListener();

	/*****************************************************************************************
	* Method used to instantiate the class. 
	* @param myModel A reference to the TreeModel providing the data for this viewer. 
	*****************************************************************************************/   
	public DIDTree(DefaultTreeModel myModel) {
		super(myModel);
		init();
	}

	/*****************************************************************************************
	* Method used to initialize this class.  Add listeners, renderers and sets the state of 
	* what to display.
	*****************************************************************************************/   
	private void init() {
		addTreeSelectionListener(mySListener);
		setRootVisible(true);
		setEditable(true);
		putClientProperty("JTree.lineStyle", "Angled");
		DIDTreeCellRenderer myCellRenderer = new DIDTreeCellRenderer();
		setCellRenderer(myCellRenderer);
		DIDTreeCellEditor myCellEditor = new DIDTreeCellEditor(this);
		setCellEditor(myCellEditor);
		addMouseListener(ml);
		expandRow(0);
	}

	MouseListener ml = new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
			int selRow = getRowForLocation(e.getX(), e.getY());
			TreePath selPath = getPathForLocation(e.getX(), e.getY());
			if ((selRow != -1) && (e.getClickCount() == 2)) {
				DIDNode node = (DIDNode)selPath.getLastPathComponent();
				DIDNode parent = (DIDNode)node.getParent();
				if (node.getNodeName().equals("Value")
				    && parent != null
				    && parent.getNodeName().equals("GT_Text")) {
					DIDUnicodeEditor ueditor = new DIDUnicodeEditor(node);
				}
			} 
     		}
	};

	/** DEL */
	public static String decodeString(String uniStr) {
		String newStr = new String();

		for(int i=0; i < uniStr.length(); i++) {
			if(uniStr.substring(i).startsWith("\\u")) {
				String temp = uniStr.substring(i+2, i+6);
				newStr += (new Character((char)Integer.parseInt(temp, 16))).toString();
				i += 5;
			} else {
				newStr += uniStr.substring(i, i+1);
			}
		}

		return newStr;
	}
}
