#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "dafs.h"

#define BUFSIZE  255

char *getEntityName(char *str, iEntityPtr EntityPtr) {
  strcpy(str, i_TypeToName(i_GetEntityType(EntityPtr)));
  if(strcmp(str, "Document") == 0) {
    strcpy(str, "Page");
  } else if(strcmp(str, "Glyph") == 0) {
    strcpy(str, "Character");
  }

  return str;
}

char *strToUpper(char *str) {
  int i;

  for(i=0; (unsigned)i < strlen(str); i++) {
    str[i] = toupper(str[i]);
  }

  return str;
}

/****/ 
char *getUText(char *buf, iUCode *utext) {
  char temp[BUFSIZE];
  int i;

  buf[0] = '\0';
  for (i=0; *(utext+i) != (iUCode)0; i++) {
    sprintf(temp, "\\u%04x", *(utext+i));
    strcat(buf, temp);
  }

  return buf;
}


/******************************************************************
* convertWord: function to change the upper case characters to lower
* case except the first character.
* ex)  ZONE -> Zone, ID -> ID(special case)
******************************************************************/
char *convertWord(char *word) {
  int i;

  if(strcmp(word, "ID") == 0) {
    return word;
  }

  for(i=1; word[i] != '\0'; i++) {
    word[i] = tolower(word[i]);
  }

  return word;
}


/******************************************************************
* convertName: function to eleminate `_' in the name.
* ex)  ZONE_ID -> ZoneID
******************************************************************/
char *convertName(char *dest, const char *name) {
  char orgName[BUFSIZE];
  char *next, *curr;

  dest[0] = '\0';
  strcpy(orgName, name);
  next = orgName;
  while(next != '\0')  {
    curr = next;
    next = strchr(curr, '_');
    if(next != NULL)  {
      *next = '\0';
      next = next + 1;
    }
    strcat(dest, convertWord(curr));
  }

//DEL//printf("%s => %s\n", name, dest);
  return dest;
}


iPropPtr findProp(iEntityPtr EntityPtr, char *propName) {
  iPropPtr prop,last_prop;

  prop=i_FirstProp(EntityPtr);
  while(prop != NULL) {
    if(strcmp(propName, i_GetPropName(prop)) == 0) {
      return prop;
    }
    last_prop=prop;
    prop=i_NextProp(EntityPtr, last_prop);
  }
  return NULL;
}

void print_properties(iEntityPtr EntityPtr, FILE *xml) {
  iPropPtr prop,last_prop;
  iPTag pdata_type;
  long longdata;
  int4 booldata;
  char *chardata;
  iUCode *ustrdata;
  char str[BUFSIZE];

  prop=i_FirstProp(EntityPtr);
  while(prop!=NULL) {
    fprintf(xml,"<%s ",convertName(str, i_GetPropName(prop)));
    pdata_type=i_GetPropType(prop);
    switch(pdata_type) {
      case iPAny:
        fprintf(xml,"Type=\"Any\" Value=\"\"/> <!-- type iPAny not handled -->\n");
        break;
      case iPNo:
        fprintf(xml,"Type=\"No\" Value=\"\"/> <!-- type iPNo not handled -->\n");
        break;
      case iPUserIData:
        fprintf(xml,"Type=\"User Data\" Value=\"\"/> <!-- type iPUserIData not handled -->\n");
        break;
      case iPBool:
        i_GetBoolProp(EntityPtr,i_GetPropName(prop),&booldata);
        fprintf(xml,"Type=\"Boolean\" Value=\"%d\"/><!-- boolean type -->\n",(int)booldata);
        break;
      case iPLong:
        i_GetLongProp(EntityPtr,i_GetPropName(prop),&longdata);
        fprintf(xml,"Type=\"Long\" Value=\"%ld\"/>\n",longdata);
        break;
      case iPString:
        i_GetStringProp(EntityPtr,i_GetPropName(prop),&chardata);
        fprintf(xml,"Type=\"String\" Value=\"%s\"/>\n",chardata);
        break;
      case iPUString:
        i_GetUStringProp(EntityPtr,i_GetPropName(prop),&ustrdata);
        fprintf(xml, "Type=\"Unicode String\" Value=\"");
        fprintf(xml, "%s", getUText(str, ustrdata));
/* DEL
        for (i=0; *(ustrdata+i) != (iUCode)0; i++) {
          fprintf(xml, "\\u%04x", *(ustrdata+i));
        }
*/
        fprintf(xml, "\"/> <!-- type iPUString not handled -->\n");
        break;
      case iPUserData:
        fprintf(xml,"Type=\"User Data\" Value=\"\"/> <!-- type iPUserData not handled -->\n");
        break;
      case iPUserPtr:
        fprintf(xml,"Type=\"User Pointer\" Value=\"\"/> <!-- type iPUserPtr not handled -->\n");
        break;
    }
    last_prop=prop;
    prop=i_NextProp(EntityPtr,last_prop);
  }
}



void printNextEntity(iEntityPtr EntityPtr, FILE *xml) {
  iEntityPtr nextPtr;
  iPropPtr prop;
  char str[BUFSIZE];
  char propName[BUFSIZE];
  char *nextId;

  nextId = "nil";
  if(i_GetEntityType(EntityPtr) == iZoneT) {
    sprintf(propName, "NEXT_SEMANTICAL_%s", strToUpper(getEntityName(str, EntityPtr)));
/*DEL*/printf("before findProp, %s = %s\n",propName, nextId);
    if((prop = findProp(EntityPtr, propName)) != NULL) {
      i_GetStringProp(EntityPtr,i_GetPropName(prop), &nextId);
/*DEL*/printf("   Zone, %s = %s\n",propName, nextId);
    }
  } else {
    if((nextPtr = i_GetNextEntity(EntityPtr)) != NULL) {
      sprintf(propName, "%s_ID", strToUpper(getEntityName(str, nextPtr)));
      if((prop = findProp(nextPtr, propName)) != NULL) {
        i_GetStringProp(nextPtr,i_GetPropName(prop), &nextId);
      }
    }
  }
/*DEL*/  if(i_GetEntityType(EntityPtr) == iZoneT) 
/*DEL*/printf("%sNext = %s\n",getEntityName(str, EntityPtr), nextId);
  fprintf(xml,"<%sNext Value=\"%s\">", getEntityName(str, EntityPtr), nextId);
  fprintf(xml,"</%sNext>\n", getEntityName(str, EntityPtr));
}


/****/ 
char *EscSpecialUChar(iUCode *utext) {
  char *result;
  char str[BUFSIZE];
  iUCode *uchar;
  int resIdx=0, specNum=0, length, i;

  length = 0;
  for (i=0; *(utext+i) != (iUCode)0; i++) {
    length++;
    switch (utext[i]) {
      case '&': case '<': case '>': case '\"':
        specNum++;
        break;
      default:
        break;
    }
  }

  result = (char *)malloc(6*(length+specNum*6+1));
  uchar = (iUCode *)malloc(sizeof(iUCode)*10);

  for(i=0; i<length; i++) {
    switch (utext[i]) {
      case '&':
/*DEL        sprintf(result+resIdx, "\\u%04x\\u%04x\\u%04x\\u%04x\\u%04x", '&', 'a', 'm', 'p', ';');*/
        uchar[0] = (iUCode)'&';
        uchar[1] = (iUCode)'a';
        uchar[2] = (iUCode)'m';
        uchar[3] = (iUCode)'p';
        uchar[4] = (iUCode)';';
        uchar[5] = (iUCode)0;
        sprintf(result+resIdx, "%s", getUText(str, uchar));
        resIdx += 5*6;
        break;

      case '<':
/*DEL        sprintf(result+resIdx, "\\u%04x\\u%04x\\u%04x\\u%04x", '&', 'l', 't', ';');*/
        uchar[0] = (iUCode)'&';
        uchar[1] = (iUCode)'l';
        uchar[2] = (iUCode)'t';
        uchar[3] = (iUCode)';';
        uchar[4] = (iUCode)0;
        sprintf(result+resIdx, "%s", getUText(str, uchar));
        resIdx += 4*6;
        break;

      case '>':
/*DEL        sprintf(result+resIdx, "\\u%04x\\u%04x\\u%04x\\u%04x", '&', 'g', 't', ';');*/
        uchar[0] = (iUCode)'&';
        uchar[1] = (iUCode)'g';
        uchar[2] = (iUCode)'t';
        uchar[3] = (iUCode)';';
        uchar[4] = (iUCode)0;
        sprintf(result+resIdx, "%s", getUText(str, uchar));
        resIdx += 4*6;
        break;

      case '\"':
/*DEL        sprintf(result+resIdx, "\\u%04x\\u%04x\\u%04x\\u%04x\\u%04x\\u%04x", '&', 'q', 'u', 'o', 't', ';');*/
        uchar[0] = (iUCode)'&';
        uchar[1] = (iUCode)'q';
        uchar[2] = (iUCode)'u';
        uchar[3] = (iUCode)'o';
        uchar[4] = (iUCode)'t';
        uchar[5] = (iUCode)';';
        uchar[6] = (iUCode)0;
        sprintf(result+resIdx, "%s", getUText(str, uchar));
        resIdx += 6*6;
        break;

      default:
        *uchar = *(utext+i);
        *(uchar+1) = (iUCode)0;
        getUText(str, uchar);
        sprintf(result+resIdx, "%s", str);
        resIdx += strlen(str);
        result[resIdx] = '\0';
        break;
    }
  }

  free(uchar);
  return result;
}

char *EscSpecialChar(char *text) {
  char *result;
  int resIdx=0, specNum=0, length, i;

  length = strlen(text);
  for(i=0; i<length; i++) {
    switch (text[i]) {
      case '&': case '<': case '>': case '\"':
        specNum++;
        break;
      default:
        break;
    }
  }

  result = (char *)malloc(length+specNum*6+1);

  for(i=0; i<length; i++) {
    switch (text[i]) {
      case '&':
        strcat(result, "&amp;");
        resIdx += 5;
        break;

      case '<':
        strcat(result, "&lt;");
        resIdx += 4;
        break;

      case '>':
        strcat(result, "&gt;");
        resIdx += 4;
        break;

      case '\"':
        strcat(result, "&quot;");
        resIdx += 6;
        break;

      default:
        result[resIdx++] = text[i];
        result[resIdx] = '\0';
        break;
    }
  }

  return result;
}

void print_contents(iEntityPtr EntityPtr, FILE *xml) {
iDTag ddata_type;
short *conf;
char *str;
iUCode *ustr;

  ddata_type=i_GetEDataType(EntityPtr);
  switch(ddata_type) {
    case iDNo:
      fprintf(xml,"<!-- content is iDNo type -->\n");
      break;
    case iDPosSet:
      fprintf(xml,"<!-- content is iDPostSet type -->\n");
      break;
    case iDString:
      fprintf(xml,"<GT_Text Type=\"String\" Value=\"");
      fprintf(xml,"%s",(str=EscSpecialChar((char *)i_GetText(EntityPtr,conf))));
      fprintf(xml,"\"></GT_Text>\n");
      free(str);
      break;
    case iDUString:
/*DEL      fprintf(xml,"<!-- content is iDUString type -->\n");*/
      fprintf(xml,"<GT_Text Type=\"Unicode String\" Value=\"");
      ustr = (iUCode *)i_GetUText(EntityPtr, conf);
      fprintf(xml,"%s",(str=EscSpecialUChar(ustr)));
      fprintf(xml,"\"></GT_Text>\n");
      free(str);
      break;
    case iD1PosSet:
    case iDUserData:
    case iDConf:
    case iDShort:
      fprintf(xml,"<!-- content data type is not handled -->\n");
    }

}

void print_bounds(iEntityPtr EntityPtr, FILE *xml) {
  iATag bdata_type;
  void *data;
  char str[BUFSIZE];

  i_GetArea(EntityPtr, &data, &bdata_type);
  fprintf(xml,"<%sCorners>\n", getEntityName(str, EntityPtr));
  switch(bdata_type) {
    case iABox:
      fprintf(xml,"<Vertex x=\"%d\" y=\"%d\"/>\n", ((iBox *)data)->x,((iBox *)data)->y);
      fprintf(xml,"<Vertex x=\"%d\" y=\"%d\"/>\n", ((iBox *)data)->x+((iBox *)data)->w,((iBox *)data)->y);
      fprintf(xml,"<Vertex x=\"%d\" y=\"%d\"/>\n" ,((iBox *)data)->x+((iBox *)data)->w,((iBox *)data)->y+((iBox *)data)->h);
      fprintf(xml,"<Vertex x=\"%d\" y=\"%d\"/>\n", ((iBox *)data)->x,((iBox *)data)->y+((iBox *)data)->h);
      break;
    case iAOrthogonList:
      fprintf(xml,"<!-- iAorthogonList -->\n");
      break;
    case iABoxList:
      fprintf(xml,"<!-- iABoxList -->\n");
      break;		
  }
  fprintf(xml,"</%sCorners>\n", getEntityName(str, EntityPtr));

}

void printRecurse(iEntityPtr EntityPtr, FILE *xml) {
  iEntityPtr last_EntityPtr;
  char str[BUFSIZE];

  while(EntityPtr!=NULL) {
    getEntityName(str, EntityPtr);
//DEL    if(strcmp(str,"Document")==0 || strcmp(str,"Zone")==0 || strcmp(str,"Line")==0 || strcmp(str,"Character")==0) {
//DEL fprintf(xml,"<%s>\n",i_TypeToName(i_GetEntityType(EntityPtr)));
    fprintf(xml,"<%s>\n",str);
    fprintf(xml,"<%sAttributes>\n", getEntityName(str, EntityPtr));
    print_properties(EntityPtr,xml);
    printNextEntity(EntityPtr,xml);
    fprintf(xml,"</%sAttributes>\n", getEntityName(str, EntityPtr));
    print_bounds(EntityPtr,xml);
    print_contents(EntityPtr,xml);
    printRecurse(i_GetChildEntity(EntityPtr),xml);
    fprintf(xml,"</%s>\n",str);
//DEL fprintf(xml,"</%s>\n",i_TypeToName(i_GetEntityType(EntityPtr)));

    last_EntityPtr=EntityPtr;
    EntityPtr=i_GetNextEntity(last_EntityPtr);
  }
}


/* add ID to the entities without ID */
void addID(iEntityPtr entity, char *parentID) {
  iEntityPtr last_EntityPtr;
  iPropPtr prop;
  char str[BUFSIZE], propName[BUFSIZE];
  char id[BUFSIZE], *tmp;
  static int zoneid = 0;
  static int lineid = 0;
  static int wordid = 0;

  while (entity != NULL)  {
    sprintf(propName, "%s_ID", strToUpper(getEntityName(str, entity)));
    switch (i_GetEntityType(entity)) {
      case iZoneT:
        if((prop = findProp(entity, propName)) == NULL) {
          sprintf(id, "%sZ%03d", parentID, zoneid);
          i_SetStringProp(entity, propName, id);
        } else {
          i_GetStringProp(entity, propName, &tmp);
          strcpy(id, tmp);
        }
        zoneid++;
        break;
      case iLineT:
        if((prop = findProp(entity, propName)) == NULL) {
          sprintf(id, "%sL%03d", parentID, lineid);
          i_SetStringProp(entity, propName, id);
        } else {
          i_GetStringProp(entity, propName, &tmp);
          strcpy(id, tmp);
        }
        lineid++;
        break;
      case iWordT:
        if((prop = findProp(entity, propName)) == NULL) {
          sprintf(id, "%sW%03d", parentID, wordid);
          i_SetStringProp(entity, propName, id);
        } else {
          i_GetStringProp(entity, propName, &tmp);
          strcpy(id, tmp);
        }
        wordid++;
        break;
      default:
        break;
    }
    addID(i_GetChildEntity(entity), id);
    last_EntityPtr=entity;
    entity = i_GetNextEntity(last_EntityPtr);
  }
}


/* main */
int main (int argc, char **argv) {
  iEntityPtr gt_doc;
  iImagePtr img;
  iDAFSError dafs_error;
  FILE *xml;
  char filename[BUFSIZE];
  char str[BUFSIZE];
  int i=0, pointPos=0;

  if(argc < 2) {
    printf("Usage: dafs2xml <DAFS file name>\n");
    printf("  ex) dafs2xml A000.DAF\n\n");
    exit(0);
  }

  i_InitLib();

  /* get the file name */
  for(i=0; argv[1][i]!='.' && argv[1][i]!='\0'; i++) {
    filename[pointPos]=argv[1][i];
    pointPos++;
    if(argv[1][i] == '/') {
      pointPos = 0;
    }
  }
  filename[pointPos] = '\0';

  /* open dafs file */
  dafs_error=i_ReadEntity(argv[1],&gt_doc);

  /* output the image file */
  strcpy(&(filename[pointPos]),".tif\0");
  img=i_GetImage(gt_doc);
  i_WriteImage(filename,img,CCITT4);
  i_DisposeImage(img);

  /* open the xml file */
  strcpy(&(filename[pointPos]),".xml\0");
  if((xml=fopen(filename,"w"))==NULL) {
    printf("[Usage]dafs2xml intput-dafs-file\n");
    printf("error opening output file:%s\n",filename);
    exit(1);
  }

  /* printout the header of the xml file */
  fprintf(xml,"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
  fprintf(xml,"<!DOCTYPE %s SYSTEM \"Trueviz.dtd\">\n", getEntityName(str, gt_doc));

  /* add ID to the entities without ID */
  addID(gt_doc, "");

  /* output page information to xml file */
  printRecurse(gt_doc,xml);


  /* close the dafs and xml files */
  fclose(xml);
  //i_DisposeEntity(gt_doc);
  i_ExitLib();

  return 0;
}
