%{

#include <stdio.h>
#include "xml2dafs.h"

extern int lineno;
void yyerror(char *s);
int yylex(void);
int yyparse(void);

%}

%union {
  char *sval;
  char cval;
}

%token  LT GT LT_SLASH GT_SLASH EQUAL LT_QUEST GT_QUEST
%token <sval> VARIABLE STRING ELE_NAME GT_TEXT VERTEX BOX COMMENT
%token <cval> CHARACTER

%%

element_list: 
	/* empty */ 
	| element_list element
	;

element:
	start_element contents end_element  { proc_element(); }
	| stand_alone_element
	| declaration
	| COMMENT
	;

stand_alone_element:  
	LT element_name attribute_list GT_SLASH { proc_stand_alone_element(); }
	;

declaration: 
	LT_QUEST VARIABLE attribute_list GT_QUEST
	;

start_element:  
	LT element_name attribute_list GT  { proc_start_element(); }
	;

end_element:  
	LT_SLASH end_element_name GT { proc_end_element(); } 
	;

contents: 
	text | element_list
	;

attribute: 
	attr_name EQUAL attr_val  { proc_attribute(); }
	;

attribute_list: 
	/* empty */ 
	| attribute_list attribute
	;

element_name:
	ELE_NAME	{ proc_element_name(yylval.sval); }
	| GT_TEXT	{ proc_gt_text_name(yylval.sval); }
	| BOX		{ proc_box_name(yylval.sval); }
	| VERTEX	{ proc_vertex_name(yylval.sval); }
	| VARIABLE	{ proc_property_name(yylval.sval); }
	;

end_element_name:
	ELE_NAME	{ proc_end_element_name(yylval.sval); }
	| GT_TEXT 	{ proc_end_gt_text_name(yylval.sval); }
	| BOX		{ proc_end_box_name(yylval.sval); }
	| VERTEX	{ proc_end_vertex_name(yylval.sval); }
	| VARIABLE	{ proc_end_property_name(yylval.sval); }
	;

attr_name: 
	VARIABLE  { proc_attr_name(yylval.sval); }
	;

attr_val: 
	STRING   { proc_attr_val(yylval.sval); }
	;

text: 
	CHARACTER | VARIABLE | CHARACTER text | VARIABLE text
	;

%%

void yyerror(char *s) {
  fprintf(stderr, "%d: %s\n", lineno, s);
}

