#include "xml2dafs.h"

char *propertyName = NULL;
char *attributeName = NULL;
char *attributeVal = NULL;
iEntityPtr rootPtr=NULL, parentPtr=NULL, currPtr=NULL;
EleType currType;

Attribute *attrList=NULL;

void freeAttrList(Attribute **attr) {
  Attribute *temp;

  while(*attr != NULL) {
    if((*attr)->name != NULL) free((*attr)->name);
    if((*attr)->val != NULL) free((*attr)->val);
    temp = (*attr);
    (*attr) = (Attribute *)(*attr)->next;
    free(temp);
  }
  (*attr) = NULL;
}


char *getAttrVal(Attribute *attrs, char *attrname) {
  Attribute *curr;

  curr = attrs;
  while(curr != NULL)  {
    if(strcmp(attrname, curr->name) == 0) {
      return curr->val;
    }
    curr = (Attribute *)curr->next;
  }
  return NULL;
}

iPTag getPropertyType(char *propName) {
  if(strcmp(propName, "Boolean") == 0)  {
    return iPBool;
  }
  else if(strcmp(propName, "Long") == 0)  {
    return iPLong;
  }
  else if(strcmp(propName, "String") == 0)  {
    return iPString;
  }
  else if(strcmp(propName, "UString") == 0)  {
    return iPUString;
  }
  else if(strcmp(propName, "User Data") == 0)  {
    return iPUserData;
  }
  else if(strcmp(propName, "User Pointer") == 0)  {
    return iPUserPtr;
  }
  return -1;
}


void proc_element()  {
}

void proc_stand_alone_element()  {
  proc_start_element();
}

void proc_start_element()  {
  char *val;

  parentPtr = currPtr;

  switch (currType) {
    case PropertyType:
      break;
      val = getAttrVal(attrList, "Type");
      switch (getPropertyType(val)) {
        case iPBool:
          val = getAttrVal(attrList, "Value");
          i_SetBoolProp(parentPtr, propertyName, atoi(val));
          break;
        case iPLong:
          val = getAttrVal(attrList, "Value");
          i_SetLongProp(parentPtr, propertyName, atol(val));
          break;
        case iPString:
          val = getAttrVal(attrList, "Value");
          i_SetStringProp(parentPtr, propertyName, val);
          break;
        case iPUString:
          break;
        case iPUserData:
          break;
        case iPUserPtr:
          break;
        default:
          break;
      }
      freeAttrList(&attrList);
      break;
    case BoxType:
      break;
    case VertexType:
/*
enum iDTag { iDNo, iDPosSet, iDString, iD1PosSet, iDUserData,
                         iDConf, iDShort, iDUString };
*/
      break;
    case GT_TextType:
      freeAttrList(&attrList);
      break;
    case ElementType:
    default:
      freeAttrList(&attrList);
      break;
  }
  
}

void proc_end_element() {
}

void proc_attribute()  {
  Attribute *new, *last;

  new = (Attribute *)malloc(sizeof(Attribute));
  new->name = attributeName;
  new->val = attributeVal;
  new->next = NULL;

  if(attrList == NULL) {
    attrList = new;
  }
  else {
    last = attrList;
    while(last->next != NULL) {
      last = (Attribute *)last->next;
    }
    (Attribute *)last->next = new;
  }
  
}

void proc_element_name(char *eleName)  {
  iTypeIndex type;
  
  if(rootPtr == NULL) {
    i_NameToType(eleName, &type);
    rootPtr = currPtr = i_NewEntity(type, parentPtr);
  }
  else {
    i_NameToType(eleName, &type);
    currPtr = i_NewEntity(type, parentPtr);
  }

  currType = ElementType;

}


void proc_property_name(char *propName)  {
  currType = PropertyType;
  propertyName = (char *)malloc(strlen(propName)+1);
  strcpy(propertyName, propName);
}


void proc_box_name(char *str)  {
  currType = BoxType;
}

 
void proc_vertex_name(char *str)  {
  currType = VertexType;
}

 
void proc_gt_text_name(char *str)  {
  currType = GT_TextType;
}


void proc_end_element_name(char *eleName)  {
}

void proc_end_property_name(char *str)  {
}


void proc_end_box_name(char *str)  {
  Attribute *curr;
  char *val;
  iBox box;
  int x1, y1, x2, y2;

  val = getAttrVal(attrList, "x");
  x1 = x2 = atoi(val);
  val = getAttrVal(attrList, "y");
  y1 = y2 = atoi(val);

  curr = attrList;
  while(curr != NULL) {
    if(strcmp("x", curr->name)==0) {
      if(atoi(curr->val) < x1) {
        x1 = atoi(curr->val);
      }
      if(atoi(curr->val) > x2) {
        x2 = atoi(curr->val);
      }
    }
    else if(strcmp("y", curr->name)==0) {
      if(atoi(curr->val) < y1) {
        y1 = atoi(curr->val);
      }
      if(atoi(curr->val) > y2) {
        y2 = atoi(curr->val);
      }
    }
    curr = (Attribute *)curr->next;
  }
  box.x = x1;
  box.y = y1;
  box.w = x2-x1;
  box.h = y2-y1;
  i_SetBox(parentPtr, box);

  freeAttrList(&attrList);
}

 
void proc_end_vertex_name(char *str)  {
}

 
void proc_end_gt_text_name(char *str)  {
}


void proc_attr_name(char *attrName)  {
  attributeName = (char *)malloc(strlen(attrName)+1);
  strcpy(attributeName, attrName);
}

void proc_attr_val(char *attrVal)  {
  attrVal = attrVal+1;   attrVal[strlen(attrVal)-1] = '\0';
  attributeVal = (char *)malloc(strlen(attrVal)+1);
  strcpy(attributeVal, attrVal);
}


int main (int argc, char **argv) {
  iImagePtr img;
  iDAFSError dafs_error;
  extern FILE *yyin;
  char filename[BUFSIZE];
  int i=0, pointPos=0;

  if(argc < 2) {
    printf("Usage: xml2dafs <XML file name>\n");
    printf("  ex) xml2dafs A000.xml\n\n");
    exit(0);
  }

  i_InitLib();

  /* get the file name */
  for(i=0; argv[1][i]!='.' && argv[1][i]!='\0'; i++) {
    filename[pointPos]=argv[1][i];
    pointPos++;
    if(argv[1][i] == '/') {
      pointPos = 0;
    }
  }

  /* open the xml file */
  strcpy(&(filename[pointPos]),".xml\0");
  if((yyin=fopen(filename,"r"))==NULL) {
    printf("[Usage]dafs2xml intput-dafs-file\n");
    printf("error opening output file:%s\n",filename);
    exit(1);
  }

  /* parse the XML file and build DAFS structure */
  yyparse();

  /* read the image file */
  strcpy(&(filename[pointPos]),".tif\0");
  dafs_error = i_ReadImage(filename,&img);
  i_SetImage(rootPtr, img);

  /* write dafs file */
  strcpy(&(filename[pointPos]),".DAF\0");
  dafs_error = i_WriteEntity(filename, rootPtr, DAFSB);

  /* close the dafs and xml files */
  //fclose(xml);
  i_ExitLib();

  return 0;
}
